﻿//
// waveファイルを解析
//
#include "parsewav.h"
#include "version.h"

#define MAX_CARRIER_PATTERN	1
static const char	*carrier_pattern[MAX_CARRIER_PATTERN][9]={
	{
		"11001100",	// 0 (600 baud)
		"00110011",	// 0 (600 baud frip)
		"10101010",	// 1 (600 baud)
		"01010101",	// 1 (600 baud frip)
		"1100",		// 0 (1200 baud)
		"0011",		// 0 (1200 baud frip)
		"1010",		// 1 (1200 baud)
		"0101",		// 1 (1200 baud frip)
		NULL
	}
};
#define IDX_PTN_600		0
#define IDX_PTN_1200	4

static const UINT8 cmt_ident[4] = { 0xff, 0x01, 0x3c, 0x00 };

// コンストラクタ
//
//
//
//
PARSEWAV::PARSEWAV()
{
	PARSEWAV(NULL);
}

PARSEWAV::PARSEWAV(wxWindow *parent)
{
	parent_window = parent;

	// パラメータ初期化
	param.range1200 = 20;
	param.range2400 = 20;
	param.freq1200 = 1200;
	param.freq2400 = 2400;
	param.reverse = 0;

	param.half_wave = 1;
	param.correct = 0;
	param.correct_type = 0;

	param.baud = 0;	// 0:600band 1:1200band
	param.auto_baud = 1;
	param.word_select = 0x04;

	param.debug_log = false;

	param.sample_rate = 3;	// 48000
	param.sample_bits = 0;	// 8bit

	param.file_split = 0;
	param.del_mhead = 0;

	//
	wave_data = new Buf_t<UINT8>(true);
	wave_correct_data = new Buf_t<UINT8>(true);
	carrier_data = new Buf_t<char>(true);
	serial_data = new Buf_t<char>(true);
	serial_new_data = new Buf_t<char>(true, true);
	binary_data = new Buf_t<UINT8>(true, true);
	freq_data = new Buf_t<int>[2];

//	buffer_size = DATA_BUFFER_SIZE - 2;

	infile_type = 0;
	outfile_type = 0;

	fio_in  = NULL;
	fio_out = NULL;
	fio_outs = NULL;
	fiolog  = NULL;

	logbuf = NULL;
	logbuf_size = -1;

	include_header = true;

	err_num = 0;

	chk_wav_mode = false;

	progbox = new ProgressBox(parent_window);
	rftypebox = new RfTypeBox(parent_window);
	maddressbox = new MAddressBox(parent_window);

	// wavファイルヘッダ(出力用)
	memcpy(outwav.head.RIFF, "RIFF", 4);
	outwav.head.file_len = 0;
	memcpy(outwav.head.WAVE, "WAVE", 4);

	memcpy(outwav.fmt.fmt, "fmt ", 4);
	outwav.fmt.format_id = 1;
	outwav.fmt.channels = 1;
	outwav.fmt.sample_rate = 48000;
	outwav.fmt.data_speed = 48000;
	outwav.fmt.block_size = 1;
	outwav.fmt.sample_bits = 8;
	outwav.fmt.fmt_size = 16;

	memcpy(outwav.data.data, "data", 4);
	outwav.data.data_len = 0;
}

// デストラクタ
//
//
//
//
PARSEWAV::~PARSEWAV()
{
	CloseDataFile();
	CloseOutFile();

	std::vector<REPORT4 *>::iterator itm = rep4.begin();
	while ( itm != rep4.end() ) {
		delete *itm;
		itm++;
	}
	rep4.clear();

	delete maddressbox;
	delete rftypebox;
	delete progbox;

	delete [] freq_data;
	delete binary_data;
	delete serial_new_data;
	delete serial_data;
	delete carrier_data;
	delete wave_correct_data;
	delete wave_data;
}

//
//
//
//
//
wxString PARSEWAV::Errmsg()
{
	wxString str = _("");

	switch(err_num) {
		case 0:
			// no error
			str = _("");
			break;
		case 1:	
			str = _("ファイルがみつかりません。");
			break;
		case 2:	
			str = _("PCMフォーマットのwavファイルではありません。");
			break;
		case 3:	
			str = _("サンプルレートは11025～48000Hzをサポートします。");
			break;
		case 4:	
			str = _("ファイルを出力できません。");
			break;
		case 5:	
			str = _("デバッグログを出力できませんが、処理を続けます。");
			break;
		case 401:	
			str = _("BASIC中間言語形式のファイルではありませんが、処理を続けます。");
			break;
		default:	
			str.Printf(_("不明なエラー: %d"), err_num);
			break;
	}
	return str;
}

// 入力ファイルを開く
// 引数:
// _TCHAR *in_file (in) ファイルパス名
// 戻り値: 0:正常 1:エラー 2:警告 -1:キャンセル終了
//
int PARSEWAV::OpenDataFile(wxString &in_file)
{
	int rc;


	infile_type = -1;
	if (check_extension(in_file, _T(".WAV"))) {
		infile_type = 0;
	} else if (check_extension(in_file, _T(".L3C"))) {
		infile_type = 1;
	} else if (check_extension(in_file, _T(".L3B"))) {
		infile_type = 2;
	} else if (check_extension(in_file, _T(".L3"))) {
		infile_type = 3;
	} else {
		infile_type = 4;
	}

	CloseDataFile();

	if (!(fio_in = _tfopen(in_file, _T("rb")))) {
		err_num= 1;
		return 1;
	}
	rc = CheckFileFormat(fio_in, in_file, infile_type);
	if (rc == 1 || rc == -1) {
		CloseDataFile();
		return rc;
	}

	infile = in_file;

	param.freq1200 = 1200;
	param.freq2400 = 2400;

	return rc;

}

// 入力ファイルを閉じる
//
//
//
//
void PARSEWAV::CloseDataFile()
{
	if (fio_in != NULL) {
		fclose(fio_in);
		fio_in = NULL;
	}
}


// 入力ファイルのフォーマットをチェックする
// 引数:
// FILE   *fp (in) 入力ファイル
// _TCHAR *in_file (in) ファイルパス名
// int     file_type (in) ファイル種類 0:waveファイル 1:l3cファイル
// 戻り値: 0:正常 1:エラーあり 2:警告あり
int PARSEWAV::CheckFileFormat(FILE *fp, const _TCHAR *in_file, int file_type) {
	int rc;

	// check wave header
	if (file_type == 0) {
		rc = check_wav_format(fp, &inwav.head, &inwav.fmt, &inwav.data);
	} else if (file_type == 1) {
		rc = check_l3c_format(fp);
	} else if (file_type == 2) {
		rc = check_l3b_format(fp);
	} else if (file_type == 3) {
		rc = check_l3_format(fp);
	} else {
		rc = check_rf_format(fp, in_file);
	}

	return rc;
}

// 出力用ファイルを開く
// 引数:
// _TCHAR *outfile (in) ファイルパス名
// 戻り値: 0:正常 1:エラーあり 2:警告あり
//
int PARSEWAV::OpenOutFile(wxString &out_file)
{
	logfile = out_file + _T(".log");

	outfile_type = -1;
	if (check_extension(out_file, _T(".WAV"))) {
		outfile_type = 0;
	} else if (check_extension(out_file, _T(".L3C"))) {
		outfile_type = 1;
	} else if (check_extension(out_file, _T(".L3B"))) {
		outfile_type = 2;
	} else if (check_extension(out_file, _T(".L3"))) {
		outfile_type = 3;
	} else {
		outfile_type = 4;
	}

	fio_out  = NULL;
	fio_outs = NULL;
	if (param.file_split && outfile_type == 4) {
		// 分割する場合
		_tcscpy(outsfileb, out_file);
		_TCHAR *p = _tcsrchr(outsfileb, '.');
		if (p != NULL) {
			_tcscpy(outsext, p);
			*p = '\0';
		} else {
			_tcscpy(outsext,_T(""));
		}
		_stprintf(outsfile, _T("%s_%03d%s"), outsfileb, 1, outsext);
		if (!(fio_outs = _tfopen(outsfile, _T("wb")))) {
			err_num = 4;
			return 1;
		}
		fclose(fio_outs);
		fio_outs = NULL;

	} else {
		// 分割しない
		if (!(fio_out = _tfopen(out_file, _T("wb")))) {
			err_num = 4;
			return 1;
		}
	}

	// デバッグログオープン
	if (param.debug_log) {
		if (!(fiolog = _tfopen(logfile, _T("w")))) {
			fiolog = NULL;
			err_num = 5;
			return 2;
		}
	}

	outfile = out_file;

	if (logbuf) {
		memset(logbuf, 0, logbuf_size);
	}
	return 0;

}

// 出力用ファイルを閉じる
//
//
//
//
void PARSEWAV::CloseOutFile()
{
	if (fio_outs != NULL) {
		fclose(fio_outs);
		fio_outs = NULL;
	}
	if (fio_out != NULL) {
		fclose(fio_out);
		fio_out = NULL;
	}
	if (fiolog != NULL) {
		fclose(fiolog);
		fiolog = NULL;
	}
}

// waveファイルのヘッダをファイルに出力する
// 引数:
// FILE   *fp (in) 入力ファイル
// 戻り値: ヘッダ長さ
//
int PARSEWAV::InitFileHeader(FILE *fp)
{
	int len = 0;
	int len_all = 0;
	len = sizeof(outwav.head);
	len_all += len;
	fwrite((void *)&outwav.head, sizeof(UINT8), len, fp);
	len = sizeof(outwav.fmt);
	len_all += len;
	fwrite((void *)&outwav.fmt, sizeof(UINT8), len, fp);
	len = sizeof(outwav.data);
	len_all += len;
	fwrite((void *)&outwav.data, sizeof(UINT8), len, fp);

	return len_all;
}

// waveファイルにファイルサイズを書き込む
// 引数:
// FILE   *fp (in) 入力ファイル
//
//
void PARSEWAV::SetFileHeader(FILE *fp)
{
	UINT32 file_size = 0;
	UINT32 riff_size = 0;
	UINT32 data_size = 0;

	fseek(fp, 0, SEEK_END);
	file_size = ftell(fp);

	riff_size = file_size - 8;

	fseek(fp, 4, SEEK_SET);
	fwrite((UINT32 *)&riff_size, sizeof(UINT32), 1, fp);
	
	data_size = file_size - 44;

	fseek(fp, 40, SEEK_SET);
	fwrite((UINT32 *)&data_size, sizeof(UINT32), 1, fp);

	return;
}

// waveサンプリングデータのレートとビットを変換してファイルに出力
// 引数:
// UINT32 rate (in) bufのサンプルレート
// size_t size (in) bufのサンプルビット
// size_t len  (in) buf長さ
// FILE   *fp  (in) wavファイル
void PARSEWAV::OutConvSampleData(void *buf, UINT32 rate, size_t size, size_t len, FILE *fp)
{
	INT16 src_buf[DATA_BUFFER_SIZE];
	size_t src_len = 0;
	size_t pos = 0;
	int new_data = 0;
	INT16 new_buf[DATA_BUFFER_SIZE];
	size_t new_len = 0;
	size_t i = 0;

	memset(src_buf, 0, sizeof(src_buf));
	for (i=0; i<(size_t)remain_buf_len; i++) {
		src_buf[i]=remain_buf[i];
	}
	src_len = remain_buf_len;

	for(i=0; i<len; i++) {
		if (size == 1) {
			// 8ビット
			src_buf[i+src_len]=(INT16)*((UINT8 *)buf + i);
		} else if (size == 2) {
			// 16ビット
			src_buf[i+src_len]=(INT16)*((INT16 *)buf + i);
		}
	}
	src_len += len;

	memset(remain_buf, 0, sizeof(remain_buf));
	remain_buf_len = 0;

	if (rate > outwav.fmt.sample_rate) {
		// rate 下げる場合
		pos = 0;
		while (pos < src_len) {
			divide = (rate + surplus) / outwav.fmt.sample_rate;
			if (pos + divide + 2 >= src_len) {
				break;
			}
			int n_surplus = (rate + surplus) % outwav.fmt.sample_rate;

			// 平均化
			new_data = 0;
			if (surplus >= 0) {
				new_data += (int)((double)src_buf[pos] * (double)((int)outwav.fmt.sample_rate - surplus) / (double)outwav.fmt.sample_rate);
				pos++;
			}
			for(i = 1; i < (size_t)divide; i++) {
				new_data += src_buf[pos];
				pos++;
			}
			if (n_surplus > 0) {
				new_data += (int)((double)src_buf[pos] * (double)n_surplus / (double)outwav.fmt.sample_rate);
			}
			new_data = (int)((double)new_data * (double)outwav.fmt.sample_rate / (double)rate); 

			surplus = n_surplus;
			new_buf[new_len] = (INT16)new_data;
			new_len++;
		}
		if (pos < src_len) {
			// 余ったデータは次回にまわす
			for(i=pos; i<src_len; i++) {
				remain_buf[i-pos]=src_buf[i];
			}
			remain_buf_len = src_len - pos;
		}
	} else if (rate < outwav.fmt.sample_rate) {
		// rate 上げる場合
		pos = 0;
		while (pos < src_len - 1) {
			divide = (outwav.fmt.sample_rate + surplus) / rate;
			surplus = (outwav.fmt.sample_rate + surplus) % rate;
			// サンプル間を線形で補完
			for(i=0; i<(size_t)divide; i++) {
				new_data = (int)src_buf[pos] + (int)i * ((int)src_buf[pos+1] - (int)src_buf[pos]) / divide;
				new_buf[new_len] = (INT16)new_data;
				new_len++;
				// バッファがいっぱいなのでファイルに出す
				if (new_len >= DATA_BUFFER_SIZE - 1) {
					out_sample_buffer(new_buf, new_len, size, fp);
					memset(new_buf, 0, sizeof(new_buf));
					new_len = 0;
				}
			}
			pos++;
		}
		// 最後のデータを次に回す
		remain_buf[0]=src_buf[src_len-1];
		remain_buf_len = 1;
	} else {
		// 同じレート
		memcpy(new_buf, src_buf, src_len * sizeof(INT16));
		new_len = src_len;
	}

	out_sample_buffer(new_buf, new_len, size, fp);

}

//
//
//
//
//
int PARSEWAV::out_sample_buffer(INT16 *new_buf, size_t new_len, size_t size, FILE *fp)
{
	INT16 new_data = 0;
	size_t i = 0;

	// 出力
	for(i=0; i<new_len; i++) {
		if (size > outwav.fmt.block_size) {
			// ビット 下げる場合
			new_data = (new_buf[i] / 256) + 128;
		} else if (size < outwav.fmt.block_size) {
			// ビット 上げる場合
			new_data = (new_buf[i] - 128) * 256;
		} else {
			new_data = new_buf[i];
		}

		if (outwav.fmt.block_size == 1) {
			fputc(new_data, fp);
		} else {
			fwrite(&new_data, sizeof(INT16), 1, fp);
		}
	}
	return 0;
}

// 変換したデータをファイルに出力
//
//
//
//
int PARSEWAV::ExportData()
{
	int rc;
	if (infile_type <= outfile_type) {
		// decode
		rc = DecodeData();
	} else if (infile_type > outfile_type) {
		// encode
		rc = EncodeData();
		// decode チェック
		if (rc == 0) rc = DecodeData();
	}
	progbox->endProgress();

	reporting();

	return rc;
}

// 波形を解析
//
//
//
//
int PARSEWAV::AnalyzeWave()
{
	int rc = 0;
	int correct_type = 0;
	int reverse = 0;

	chk_wav_mode = true;

	outfile_type = 5;	// 出力ファイルなし

	std::vector<REPORT4 *>::iterator itm = rep4.begin();
	while ( itm != rep4.end() ) {
		delete *itm;
		itm++;
	}
	rep4.clear();
	rep4_itm = NULL;

	half_wave = 1;

	memset(save_data_name, 0, sizeof(save_data_name));
	save_data_name[8]=0x7f;
	save_data_name[9]=0x7f;

	if (logbuf) {
		memset(logbuf, 0, logbuf_size);
	}

	memset(&st_chkwav, 0, sizeof(st_chkwav));

	progbox->initProgress(0, 100);

	for(int i=0; i < 2; i++) {
		st_chkwav.num = i;
		// fseekする
		fseek(fio_in, 0, SEEK_SET);
		sample_pos = 0;
		write_pos = 0;
		sample_ms = 0;

		check_wav_format(fio_in, &inwav.head, &inwav.fmt, &inwav.data);

		wave_data->init();
		wave_correct_data->init();
		carrier_data->init();
		serial_data->init();
		serial_new_data->init();
		binary_data->init();

		st_chkwav.analyze_num = inwav.fmt.sample_rate * 10;	// 10秒解析

		phase2_frip = 0;
		phase3_baud = 0;
		phase2n_baud_count = 0;

		memset(rep_p1_sample_num, 0, sizeof(rep_p1_sample_num));
		rep_p2_error_num = 0;
		rep_p3_error_num = 0;

		correct_type = i % 2;
		reverse = 0;

		phase1 = 0;
		phase2 = 11;
		phase2n = 0;
		phase3 = 0;
		phase4 = 0;

		decode_phase1(wave_data, wave_correct_data, carrier_data, serial_data, serial_new_data, binary_data, 0, correct_type, reverse);
		st_chkwav.sample_num[i][0]=rep_p1_sample_num[0];
		st_chkwav.sample_num[i][1]=rep_p1_sample_num[1];
		st_chkwav.sample_num[i][2]=rep_p1_sample_num[2];
		st_chkwav.us1200avg[i] = st_lamda.us1200avg;
		st_chkwav.us2400avg[i] = st_lamda.us2400avg;
	}

	progbox->endProgress();

	// 一番適切な組み合わせを探す
	long long long_data = 0;
	long long mid_data = sample_num;
	int long_num = -1;
	int mid_num = 0;
	int best_num = 0;
	// sin or cos ?
	for(int i=1; i >= 0; i--) {
		// long波の個数
		if (long_data < (st_chkwav.sample_num[i][0] / 10)) {
			// 最大値
			long_data = (st_chkwav.sample_num[i][0] / 10);
			long_num = i;
		}
		if (mid_data >= (st_chkwav.sample_num[i][2] / 10)) {
			// middle波の個数
			mid_data = (st_chkwav.sample_num[i][2] / 10);
			mid_num = i;
		} else if (mid_data == (st_chkwav.sample_num[i][2] / 10)) {
			// 同じ値がある場合
			mid_num = -1;
		}
	}
	if (mid_num >= 0) {
		best_num = mid_num;
	} else {
		best_num = long_num;
	}

	// ボーレート判定
	if ((double)st_chkwav.baud_num[best_num][0] < (double)st_chkwav.baud_num[best_num][1]) {
		// 1200ボー
		param.baud = 1;
	} else {
		// 600ボー
		param.baud = 0;
	}

	// 波正逆判定
	if (st_chkwav.rev_num[best_num][0] < st_chkwav.rev_num[best_num][1]) {
		// 逆
		param.reverse = 1;
	} else {
		// 正
		param.reverse = 0;
	}

	// パラメータをセット
	param.correct = 1;
	param.correct_type = best_num % 2;

	param.freq1200 = (int)(1000000.0 / st_chkwav.us1200avg[best_num] /  (1 << half_wave));
	param.freq2400 = (int)(1000000.0 / st_chkwav.us2400avg[best_num] /  (1 << half_wave));

	// レポート
	reporting_analyze();

	chk_wav_mode = false;

	return rc;
}

// 音データからバイナリデータに変換
//
//
//
//
int PARSEWAV::DecodeData()
{
	wave_data->init();
	wave_correct_data->init();
	carrier_data->init();
	serial_data->init();
	serial_new_data->init();
	binary_data->init();

	phase2_frip = 0;

	phase3_baud = 0;
	phase2n_baud_count = 0;

	half_wave = param.half_wave;

	memset(rep_p1_sample_num, 0, sizeof(rep_p1_sample_num));
	rep_p2_error_num = 0;
	rep_p3_error_num = 0;

	std::vector<REPORT4 *>::iterator itm = rep4.begin();
	while ( itm != rep4.end() ) {
		delete *itm;
		itm++;
	}
	rep4.clear();
	rep4_itm = NULL;

	memset(save_data_name, 0, sizeof(save_data_name));
	save_data_name[8]=0x7f;
	save_data_name[9]=0x7f;

	memset(&inwav, 0, sizeof(inwav));

	// fseekする
	fseek(fio_in, 0, SEEK_SET);
	sample_pos = 0;
	write_pos = 0;
	sample_ms = 0;

	if (infile_type == 0) {
		check_wav_format(fio_in, &inwav.head, &inwav.fmt, &inwav.data);
	}

	// waveファイルのヘッダを出力
	if (outfile_type == 0 && outfile_type >= infile_type) {
		outwav.fmt.channels = 1;
		outwav.fmt.sample_rate = get_sample_rate(param.sample_rate);
		outwav.fmt.sample_bits = (param.sample_bits == 0 ? 8 : 16);
		outwav.fmt.block_size = (param.sample_bits == 0 ? 1 : 2);
		outwav.fmt.data_speed = outwav.fmt.sample_rate * outwav.fmt.block_size;

		remain_buf_len = 0;
		memset(remain_buf, 0, sizeof(remain_buf));
		divide = 0;
		surplus = 0;

		InitFileHeader(fio_out);
	}

	progbox->initProgress(0, 100);

	if (infile_type == 0) {
		phase1 = 0;
		phase2 = 0;
		phase2n = 0;
		phase3 = 0;
		phase4 = 0;
		decode_phase1(wave_data, wave_correct_data, carrier_data, serial_data, serial_new_data, binary_data, 0);
	} else if (infile_type == 1) {
		phase1 = 11;
		phase2 = 0;
		phase2n = 0;
		phase3 = 0;
		phase4 = 0;
		decode_phase1(wave_data, wave_correct_data, carrier_data, serial_data, serial_new_data, binary_data, 11);
	} else if (infile_type == 2) {
		phase1 = 11;
		phase2 = 21;
		phase2n = 0;
		phase3 = 0;
		phase4 = 0;
		carrier_data->last = true;
		decode_phase2(carrier_data, serial_data, serial_new_data, binary_data, 21);
	} else if (infile_type == 3) {
		phase1 = 11;
		phase2 = 21;
		phase2n = 0;
		phase3 = 31;
		phase4 = 0;
		serial_data->last = true;
		decode_phase3(serial_data, binary_data, 31);
	}

	// waveファイルのヘッダを更新
	if (outfile_type == 0 && outfile_type >= infile_type) {
		SetFileHeader(fio_out);
	}

	return 0;
}

// バイナリデータから音データに変換
//
//
//
//
int PARSEWAV::EncodeData()
{
	wave_data->init();
	carrier_data->init();
	serial_data->init();
	serial_new_data->init();
	binary_data->init();

	phase2_frip = 0;
	phase3_baud = 0;
	phase2n_baud_count = 0;

	memset(rep_p1_sample_num, 0, sizeof(rep_p1_sample_num));
	rep_p2_error_num = 0;
	rep_p3_error_num = 0;

	// 実ファイルの情報をセット
	memset(save_data_name, 0, sizeof(save_data_name));
	if (infile_type == 4) {
		rftypebox->getRfDataName(save_data_name);
		save_data_name[8]=rftypebox->getRfDataFormat();
		save_data_name[9]=rftypebox->getRfDataType();
		save_data_name[10]=rftypebox->getRfDataType();

		// マシン語の場合
		if (save_data_name[8] == 2 && maddressbox->getValid() != true) {
			// ヘッダがない場合、情報入力を促す
			if (ShowMAddressBox(true) == false) {
				return -1;
			}
		}

		if (include_header) {
			sample_num = sample_num_orig;
		} else {
			sample_num = sample_num_orig + 10;
		}
	}

	fseek(fio_in, 0, SEEK_SET);
	sample_pos = 0;
	write_pos = 0;
	sample_ms = 0;

	// waveファイルのヘッダを出力
	if (outfile_type == 0) {
		outwav.fmt.channels = 1;
		outwav.fmt.sample_rate = get_sample_rate(param.sample_rate);
		outwav.fmt.sample_bits = (param.sample_bits == 0 ? 8 : 16);
		outwav.fmt.block_size = (param.sample_bits == 0 ? 1 : 2);
		outwav.fmt.data_speed = outwav.fmt.sample_rate * outwav.fmt.block_size;

		remain_buf_len = 0;
		memset(remain_buf, 0, sizeof(remain_buf));
		divide = 0;
		surplus = 0;

		InitFileHeader(fio_out);
	}

	progbox->initProgress(0, 100);

	if (infile_type == 1) {
		phase4 = 0;
		phase3 = 20;
		phase2n = 0;
		phase2 = 10;
		phase1 = 11;
		encode_phase2(serial_data, carrier_data, 10, true);
	} else if (infile_type == 2) {
		phase4 = 0;
		phase3 = 20;
		phase2n = 0;
		phase2 = 21;
		phase1 = 11;
		encode_phase3(binary_data, serial_data, carrier_data, 20);
	} else if (infile_type == 3) {
		phase4 = 0;
		phase3 = 30;
		phase2n = 0;
		phase2 = 21;
		phase1 = 11;
		encode_phase3(binary_data, serial_data, carrier_data, 30);
	} else if (infile_type == 4) {
		phase4 = 0;
		phase3 = 40;
		phase2n = 0;
		phase2 = 21;
		phase1 = 11;
		encode_phase4(binary_data, serial_data, carrier_data);
	}

	// waveファイルのヘッダを更新
	if (outfile_type == 0) {
		SetFileHeader(fio_out);
	}

	return 0;
}

int PARSEWAV::get_sample_rate(int num) {
	int rate = 48000;

	switch(num) {
		case 0:
			rate = 11025;
			break;
		case 1:
			rate = 22050;
			break;
		case 2:
			rate = 44100;
			break;
	}
	return rate;
}

// WAVEファイルのフォーマットをチェックする
// 引数:
// FILE         *fp     (in) 入力ファイル
// wav_header_t *header (out) ヘッダ
// wav_data_t   *data   (out) データ
// 戻り値: 0: OK  1:エラー
int PARSEWAV::check_wav_format(FILE *fp, wav_header_t *head, wav_fmt_chank_t *fmt, wav_data_chank_t *data)
{
	char buf[10];
	long offset = 0;
	long fpos_data = 0;
	wav_unknown_chank_t unk;

	memset(head, 0, sizeof(wav_header_t));
	memset(fmt, 0, sizeof(wav_fmt_chank_t));
	memset(data, 0, sizeof(wav_data_chank_t));

	fread(head, sizeof(wav_header_t), 1, fp);
	if(memcmp(head->RIFF,"RIFF",4) != 0 || memcmp(head->WAVE,"WAVE",4) != 0) {
		// this is not wave format !!!
		err_num = 2;
		return 1;
	}

	for (int i=0; i<10; i++) {
		fread(buf, sizeof(char), 4, fp);
		if (memcmp(buf, "fmt ", 4) == 0) {
			// fmt chank
			fseek(fp, -4, SEEK_CUR);
			fread(fmt, sizeof(wav_fmt_chank_t), 1, fp);
			if (fmt->format_id != 1) {
				// this is not pcm format !!!
				err_num = 2;
				return 1;
			}

			// 11025 〜 48000Hz
			if (fmt->sample_rate < 11025 || 48000 < fmt->sample_rate) {
				err_num = 3;
				return 1;
			}

			if (fpos_data != 0) break;

			offset = 8 + fmt->fmt_size - sizeof(wav_fmt_chank_t);

		} else if (memcmp(buf, "data", 4) == 0) {
			// data chank
			fseek(fp, -4, SEEK_CUR);
			fread(data, sizeof(wav_data_chank_t), 1, fp);
			fpos_data = ftell(fp);

			if (fmt->format_id != 0) break;

			offset = data->data_len;

		} else {
			// unknown chank
			fseek(fp, -4, SEEK_CUR);
			fread(&unk, sizeof(wav_unknown_chank_t), 1, fp);

			offset = unk.len;
		}
		fseek(fp, offset, SEEK_CUR);
	}

	if (fpos_data == 0 || fmt->format_id != 1) {
		// this is not pcm format !!!
		err_num = 2;
		return 1;
	}

	fseek(fp, fpos_data, SEEK_SET);

	sample_num = (data->data_len) / fmt->channels;
	if(fmt->sample_bits == 16) {
		sample_num /= 2;
	}
	sample_num_orig = sample_num;
	sample_pos = 0;
	sample_ms = 0;

	return 0;
}

// l3cファイルのフォーマットをチェック（チェックしていないが）
//
//
//
//
int PARSEWAV::check_l3c_format(FILE *fp)
{
	fseek(fp, 0, SEEK_END);
	sample_num = ftell(fp);
	fseek(fp, 0, SEEK_SET);

	sample_num_orig = sample_num;
	sample_pos = 0;
	sample_ms = 0;

	return 0;
}

// l3bファイルのフォーマットをチェック（チェックしていないが）
//
//
//
//
int PARSEWAV::check_l3b_format(FILE *fp)
{
	fseek(fp, 0, SEEK_END);
	sample_num = ftell(fp);
	fseek(fp, 0, SEEK_SET);

	sample_num_orig = sample_num;
	sample_pos = 0;
	sample_ms = 0;

	return 0;
}

// l3ファイルのフォーマットをチェック（チェックしていないが）
//
//
//
//
int PARSEWAV::check_l3_format(FILE *fp)
{
	fseek(fp, 0, SEEK_END);
	sample_num = ftell(fp);
	fseek(fp, 0, SEEK_SET);

	sample_num_orig = sample_num;
	sample_pos = 0;
	sample_ms = 0;

	return 0;
}

// 実ファイルのフォーマットをチェック
//
//
//
//　-1 キャンセル終了
int PARSEWAV::check_rf_format(FILE *fp, const _TCHAR *in_file)
{
	int rc;
	_TCHAR bname[_MAX_PATH];

	base_name(in_file, bname);

	// ファイルの種類を選択
	rc = rftypebox->showRftypeBox(bname, true);
	if (rc != 1) {
		// cancel button
		return -1;
	}

	return get_first_rf_data(fp);
}

//
//
//
//
//
int PARSEWAV::get_first_rf_data(FILE *fp)
{
	int rc = 0;
	int dfmt;
	int dtype;
	UINT addr;

	include_header = true;

	fseek(fp, 0, SEEK_END);
	sample_num = ftell(fp);
	fseek(fp, 0, SEEK_SET);

	sample_num_orig = sample_num;
	sample_pos = 0;
	sample_ms = 0;

	dfmt = rftypebox->getRfDataFormat();
	dtype = rftypebox->getRfDataType();
	switch(dfmt) {
		case 0:	// basic
			if (dtype != 0xff) { // 中間言語形式の場合
				// ファイルの最初を読み込む
				fread(rf_header, sizeof(UINT8), sizeof(rf_header), fp);
				if (rf_header[0] != 0xff && rf_header[0] != 0xfe) {
					// maybe not have header
					err_num = 401;
					rc = 2;
				}
			}
			break;

		case 2: // machine 

			// ファイルの最初/最後を読み込む
			fread(rf_header, sizeof(UINT8), sizeof(rf_header), fp);
			fseek(fp, -5, SEEK_END);
			fread(rf_footer, sizeof(UINT8), sizeof(rf_footer), fp);

			if (rf_header[0] != 0x00 || rf_footer[0] != 0xff) {
				// maybe not have header and footer
				include_header = false;
				maddressbox->setValid(false);
				maddressbox->setDataSize((UINT)sample_num);
				// ダイアログ表示
				ShowMAddressBox(true);
			} else {
				maddressbox->setValid(true);
				addr = rf_header[3] * 256 + rf_header[4];
				maddressbox->setStartAddr(addr);
				addr = rf_header[1] * 256 + rf_header[2];
				maddressbox->setDataSize(addr);
				addr = rf_footer[3] * 256 + rf_footer[4];
				maddressbox->setExecAddr(addr);
				maddressbox->setIncHeader(include_header);
			}
			break;
	}

	fseek(fp, 0, SEEK_SET);

	return rc;
}


// wavファイルから１サンプル読む
// size : 1  : UINT8で返す  2 : INT16で返す
// reverse: 1: 波形を反転
//
//
int PARSEWAV::get_wav_sample(int size, int reverse)
{
	int l = 0;
	int h = 0;
	
	// 1サンプルを得る
	l = fgetc(fio_in);
	if(inwav.fmt.sample_bits == 16) {
		// INT16の場合
		h = fgetc(fio_in);
		if (h >= 128) {
			h = h - 256;
		}
		if (reverse) {
			h *= -1;
			if (h >= 128) h = 127;
			l *= -1;
		}
	} else {
		if (reverse) {
			l = 256 - l;
			if (l >= 256) l = 255;
		}
	}
	// ステレオの場合、左側を無視する
	for(int i = 2; i <= inwav.fmt.channels; i++) {
		fgetc(fio_in);
		if(inwav.fmt.sample_bits == 16) {
			fgetc(fio_in);
		}
	}
	sample_pos++;
	sample_ms = (UINT32)(1000.0 * (double)sample_pos / (double)inwav.fmt.sample_rate);

	if (size == 1) {
		if(inwav.fmt.sample_bits == 16) {
			// INT16 -> UINT8
			return ((h + 128) & 0xff);
		} else {
			return (l & 0xff);
		}
	} else {
		if(inwav.fmt.sample_bits == 16) {
			return h * 256 + l;
		} else {
			// UINT8 -> INT16
			return (l - 128) * 256;
		}
	}
}
int PARSEWAV::get_wav_sample(int reverse)
{
	return get_wav_sample(2, reverse);
}

// wavファイルから１サンプル読む
// w_data (in/out) サンプルデータ(追記していく)
// reverse: 1: 波形を反転
// 戻り値: 読み込んだデータの長さ
//
int PARSEWAV::get_wav_sample(Buf_t<UINT8> *w_data, int reverse)
{
	int l;

	while(w_data->isFull(0) != true && sample_pos < sample_num) {
		l = get_wav_sample(1, reverse);

		w_data->add((UINT8)l, sample_ms);
	}
	if (sample_pos >= sample_num) {
		w_data->last = true;
	}
	return w_data->w_pos;
}

// l3cファイル(搬送波ビットデータ)から１データ読む
// c_data (in/out) 搬送波データ(追記していく)
// 戻り値: 読み込んだデータの長さ
//
//
int PARSEWAV::get_l3c_sample(Buf_t<char> *c_data)
{
//	int pos = 0;
	int l;

//	pos = strlen(data);
	while(c_data->isFull(0) != true && sample_pos < sample_num) {
		l = fgetc(fio_in);
		l &= 0xff;

		sample_pos++;
		sample_ms = (UINT32)(1000.0 * (double)sample_pos / 4800.0);
		if (l == '\r' || l == '\n') {
			continue;
		}
		c_data->add(l, sample_ms);
	}
	if (sample_pos >= sample_num) {
		c_data->last = true;
	}
	return c_data->w_pos;
}

// l3bファイルから１データ読む
//
//
//
//
int PARSEWAV::get_l3b_sample(Buf_t<char> *s_data)
{
//	int pos = 0;
	int l;

//	pos = strlen(data);
	while(s_data->isFull(0) != true && sample_pos < sample_num) {
		l = fgetc(fio_in);
		l &= 0xff;

		sample_pos++;
		sample_ms = (UINT32)(1000.0 * (double)sample_pos / (param.baud ? 1200.0 : 600.0));
		if (l == '\r' || l == '\n') {
			continue;
		}
		s_data->add(l, sample_ms);
	}
	if (sample_pos >= sample_num) {
		s_data->last = true;
	}
	return s_data->w_pos;
}

// l3ファイルからデータ読む
//
//
//
//
int PARSEWAV::get_l3_sample(Buf_t<UINT8> *b_data)
{
	int pos = 0;

	pos = fread(&(b_data->data[b_data->w_pos]), sizeof(UINT8), b_data->size-(b_data->w_pos), fio_in);
	sample_pos += pos;
	(b_data->w_pos) += pos;

	if (sample_pos >= sample_num) {
		b_data->last = true;
	}
	return (b_data->w_pos);
}

// l3ファイルから１データ読む
//
//
//
//
UINT8 PARSEWAV::get_l3_sample()
{
	int l;

	l = fgetc(fio_in);

	sample_pos++;

	return (l & 0xff);
}

// 実ファイルから１データ読む
//
//
//
//
UINT8 PARSEWAV::get_rf_sample()
{
	int l;

	if (save_data_name[8] == 2) {
		// マシン語
		if (sample_pos < 5) {
			if (include_header) l = fgetc(fio_in);
			l = rf_header[sample_pos];
		} else if (sample_num - sample_pos <= 5) {
			if (include_header) l = fgetc(fio_in);
			l = rf_footer[5 - sample_num + sample_pos];
		} else {
			l = fgetc(fio_in);
		}
	} else {
		l = fgetc(fio_in);
	}
	sample_pos++;

	return (l & 0xff);
}

void PARSEWAV::init_dft(int type)
{
	int half = samples / 2;
	int n, k;

	for(k = 0; k < 2; k++) {
		for(n = 0; n < samples; n++) {
			if (type == 0) {
				h[k][n] = cos(-2.0 * M_PI * (k + 1) * (n - half) / samples);
			} else {
				h[k][n] = sin(-2.0 * M_PI * (k + 1) * (n - half) / samples);
			}
		}
	}
	return;
}


// フーリエ変換
// 1200Hz, 2400Hzの正弦波or余弦波を合成して補正する
//
//
//
int PARSEWAV::calcrate_dft(Buf_t<UINT8> *w_data, Buf_t<UINT8> *wc_data)
{
	int k;
	int n;
	int p;
	int wc;
	double dat;
	Buf_t<int> *f_data;

	int half = samples / 2;

	for (k = 0; k < 2; k++) {
		f_data = &freq_data[k];
		f_data->init();
	}
	for (p = half; p < (w_data->w_pos - half); p++) {
		for (k = 0; k < 2; k++) {
			f_data = &freq_data[k];

			dat = 0.0;
			for (n = p - half; n < (p + half); n++) {
				dat += (double)((int)w_data->data[n] - 128) * h[k][n - p + half];
			}
			f_data->add((int)(dat * 1000.0 / samples), 0);
		}
		wc = (freq_data[0].data[p - half] + freq_data[1].data[p - half]) / 1000;
		if (wc >= 128) wc = 127;
		if (wc < -128) wc = -128;
		wc_data->add((UINT8)(wc + 128), w_data->get_ms(0));
		w_data->r_pos++;
		w_data->total_r_pos++;
		if (wc_data->isFull(0)) {
			break;
		}
	}
#if 0
	for (p = 0; p < (wc_data->w_pos); p++) {
//		if (param.debug_log) {
		sprintf(cbuff,"%d,%d,%d,%d,%d,%d\n"
			, (w_data->data[p + half] - 128) * 1000
			, freq_data[0].re[p]
			, freq_data[1].re[p]
			, freq_data[2].re[p]
			, (w_data->data[p + half] - 128)
			, (wc_data->data[p + wc_data->start_pos] - 128)
		);
		if (fiolog) fwrite((void *)cbuff, strlen(cbuff), 1, fiolog);
//		}
	}
#endif
	return wc_data->w_pos;
}

// 2400Hzと1200Hzの波を見つけてビットデータに変換
// 引数:
// Buf_t<UINT8> *w_data          (in) WAVサンプルデータ
// char    *bit_data        (out) "10": 2400Hz "1100":1200Hz "?HHLL":Too long
// 戻り値: WAVサンプルデータ
int PARSEWAV::decode_to_carrier_wave_bn(Buf_t<UINT8> *w_data, Buf_t<char> *c_data) 
{
	char bit_data[10];

	int data = 0;
	UINT8  dat;
	UINT32 ms;
	bool hi = false;
	bool found = false;

	double x0,lamda;

	// find next trigger point
	while(w_data->isEOB(0) != true) {
		w_data->get(dat, ms);
		data = (int)dat - 128;

		if (st_pa_carr.wav_prev < 0 && data >= 0) {
			// l -> h になる位置から波長(us)を求める
			hi = true;
			x0 = - (st_pa_carr.wav_prev * st_lamda.us_delta / (data - st_pa_carr.wav_prev));
			lamda = st_pa_carr.x0_prev + (st_pa_carr.sample_cnt * st_lamda.us_delta) + x0;
			// 波長が短すぎる場合はノイズと判断して処理継続
			if (lamda >= 105.0) {
				st_pa_carr.x0_prev = st_lamda.us_delta - x0;
				st_pa_carr.odd = 1 - st_pa_carr.odd;
				found = true;
				break;
			}
		} else if (st_pa_carr.wav_prev > 0 && data <= 0 && half_wave == 1) {
			// h -> l になる位置から波長(us)を求める
			hi = false;
			x0 = - (st_pa_carr.wav_prev * st_lamda.us_delta / (data - st_pa_carr.wav_prev));
			lamda = st_pa_carr.x0_prev + (st_pa_carr.sample_cnt * st_lamda.us_delta) + x0;
			// 波長が短すぎる場合はノイズと判断して処理継続
			if (lamda >= 105.0) {
				st_pa_carr.x0_prev = st_lamda.us_delta - x0;
				st_pa_carr.odd = 1 - st_pa_carr.odd;
				found = true;
				break;
			}
		}
		st_pa_carr.sample_cnt++;
		st_pa_carr.wav_prev = data;
	}
	if (found != true) {
		return data;
	}

	if (half_wave == 0) {
		// 全波
		if (st_lamda.us1200min <= lamda && lamda <= st_lamda.us1200max) {
			// long (half wave)
			strcpy(bit_data,"1100");
			st_pa_carr.carr_prev = 4;
			st_lamda.us1200avg = (st_lamda.us1200avg + lamda) / 2.0;

			rep_p1_sample_num[0]++;
		}
		else if (st_lamda.us2400min <= lamda && lamda <= st_lamda.us2400max) {
			// short (half wave)
			strcpy(bit_data,"10");
			st_pa_carr.carr_prev = 2;
			st_lamda.us2400avg = (st_lamda.us2400avg + lamda) / 2.0;

			rep_p1_sample_num[1]++;
		}
		else if (st_lamda.us1800 <= lamda && lamda < st_lamda.us1200min) {
			// middle long 
			strcpy(bit_data,"?1100");
			st_pa_carr.carr_prev = 4;
			st_lamda.us1800avg = (st_lamda.us1800avg + lamda) / 2.0;

			rep_p1_sample_num[2]++;
		}
		else if (st_lamda.us2400max < lamda && lamda < st_lamda.us1800) {
			// middle short
			strcpy(bit_data,"?10");
			st_pa_carr.carr_prev = 2;
			st_lamda.us1800avg = (st_lamda.us1800avg + lamda) / 2.0;

			rep_p1_sample_num[2]++;
		}
		else if (st_lamda.us1200max < lamda) {
			// too long 
			strcpy(bit_data,"?HHLL");
			st_pa_carr.carr_prev = 5;
			rep_p1_sample_num[3]++;
		}
		else if (lamda < st_lamda.us2400min) {
			// too short
			strcpy(bit_data,"?HL");
			st_pa_carr.carr_prev = 1;
			rep_p1_sample_num[4]++;
		}
		else {
			// error
			strcpy(bit_data,"??");
			st_pa_carr.carr_prev = 0;
			rep_p1_sample_num[5]++;
		}
	} else {
		// 半波
		if (st_lamda.us1200min <= lamda && lamda <= st_lamda.us1200max) {
			// long (half wave)
			if (hi) {
				strcpy(bit_data,"00");
				if (st_pa_carr.carr_prev != 4) st_pa_carr.odd = 1;
				st_pa_carr.carr_prev = -4;
			} else {
				strcpy(bit_data,"11");
				if (st_pa_carr.carr_prev != -4) st_pa_carr.odd = 1;
				st_pa_carr.carr_prev = 4;
			}
			st_lamda.us1200avg = (st_lamda.us1200avg + lamda) / 2.0;

			rep_p1_sample_num[0]++;
		}
		else if (st_lamda.us2400min <= lamda && lamda <= st_lamda.us2400max) {
			// short (half wave)
			if (hi) {
				strcpy(bit_data,"0");
				if (st_pa_carr.carr_prev != 2) st_pa_carr.odd = 1;
				st_pa_carr.carr_prev = -2;
			} else {
				strcpy(bit_data,"1");
				if (st_pa_carr.carr_prev != -2) st_pa_carr.odd = 1;
				st_pa_carr.carr_prev = 2;
			}
			st_lamda.us2400avg = (st_lamda.us2400avg + lamda) / 2.0;

			rep_p1_sample_num[1]++;
		}
		else if (st_lamda.us2400max < lamda && lamda < st_lamda.us1200min) {
			// middle 
			if (hi) {
				if ((st_pa_carr.odd == 0 && st_pa_carr.carr_prev == 4)
				 || (st_pa_carr.odd == 1 && st_pa_carr.carr_prev == 2)) {
					strcpy(bit_data,"?00");
					st_pa_carr.carr_prev = -4;
				} else {
					strcpy(bit_data,"?0");
					st_pa_carr.carr_prev = -2;
				}
			} else {
				if ((st_pa_carr.odd == 0 && st_pa_carr.carr_prev == -4)
				 || (st_pa_carr.odd == 1 && st_pa_carr.carr_prev == -2)) {
					strcpy(bit_data,"?11");
					st_pa_carr.carr_prev = 4;
				} else {
					strcpy(bit_data,"?1");
					st_pa_carr.carr_prev = 2;
				}
			}
			st_lamda.us1800avg = (st_lamda.us1800avg + lamda) / 2.0;

			rep_p1_sample_num[2]++;
		}
		else if (st_lamda.us1200max < lamda) {
			// too long 
			if (hi) {
				strcpy(bit_data,"?LL");
				st_pa_carr.carr_prev = -5;
			} else {
				strcpy(bit_data,"?HH");
				st_pa_carr.carr_prev = 5;
			}
			rep_p1_sample_num[3]++;
		}
		else if (lamda < st_lamda.us2400min) {
			// too short
			if (hi) {
				strcpy(bit_data,"?L");
				st_pa_carr.carr_prev = -1;
			} else {
				strcpy(bit_data,"?H");
				st_pa_carr.carr_prev = 1;
			}
			rep_p1_sample_num[4]++;
		}
		else {
			// error
			strcpy(bit_data,"??");
			st_pa_carr.carr_prev = 0;

			rep_p1_sample_num[5]++;
		}
	}
	// データ追加
	if (bit_data[0] != '?') {
		// OK data
		c_data->adds(bit_data, ms);
	} else {
		// NG data
		c_data->adds(&bit_data[1], ms);

		if (param.debug_log > 1) {
			// デバッグログ
			sprintf(cbuff, "p1 w:%12d(%s) c:%12d: %s\n" 
				, w_data->total_r_pos
				, get_time_cstr(ms)
				, c_data->total_w_pos
				, bit_data);
				if (fiolog) fwrite((void *)cbuff, strlen(cbuff), 1, fiolog);
		}
	}

	st_pa_carr.wav_prev = data;
	st_pa_carr.sample_cnt = 0;

	return data;
}

// 一致するパターンを探す
// 引数:
// Buf_t<char> *c_data (in) 搬送波
// int       baud   (in) 600baud:0 1200baud:1
// int      *frip   (out) 1->0->1->0: 0 0->1->0->1:1
// 戻り値: スタート位置 / -1 なし
int PARSEWAV::find_start_carrier_bit(Buf_t<char> *c_data, Buf_t<char> *s_data, int baud, int *frip)
{
	int pos1[4];
	int pos = -1;
	int idx_ptn = (baud & 1) * IDX_PTN_1200;	// 600:0 1200:4
	int best_num = -1;
	int best_pos = c_data->size;
	int i;
	int ptn;
    char bit = '?';
	UINT32 ms;

	// 一致するパターンを探す
	for(ptn=0; ptn<MAX_CARRIER_PATTERN; ptn++) {
		for(i=0; i<4; i++) {
			pos1[i]=c_data->find(0, carrier_pattern[ptn][idx_ptn + i]);
		}

		for(i=0; i<4; i++) {
			if (pos1[i] >= 0 && best_pos > pos1[i]) {
				best_pos = pos1[i];
				best_num = i;
			}
		}

		if (best_num < 0) {
			// なし
			break;
		}

		pos = best_pos + strlen(carrier_pattern[ptn][idx_ptn + best_num]);

		if ((best_num % 2) == 0) {
			(*frip) = 0;
		} else {
			(*frip) = 1;
		}
		if ((best_num / 2) == 0) {
			bit = '0';
		} else {
			bit = '1';
		}

		break;
	}

	if (pos >= 0) {
		// データ有り
		ms = c_data->get_ms(best_pos);

		if (param.debug_log > 1) {
			// デバッグログ
			sprintf(cbuff, "p2 c:%12d(%s) w_pos:%d pos:%d frip:%d ("
				, c_data->total_r_pos
				, get_time_cstr(ms)
				, s_data->total_w_pos
				, best_pos, *frip);
			strncat(cbuff, c_data->curptr(best_pos), pos - best_pos);
			strcat(cbuff, ")\n");
				if (fiolog) fwrite((void *)cbuff, strlen(cbuff), 1, fiolog);
		}

		c_data->r_pos += pos;
		c_data->total_r_pos += pos;

		s_data->add(bit, ms);
	} else {
		// データなし
		c_data->total_r_pos += c_data->w_pos - c_data->r_pos;
		c_data->r_pos = c_data->w_pos;
	}

	return pos;
}

// 搬送波(2400/1200Hz)のデータからシリアルデータを１ビット取り出す
// 引数:
// Buf_t<char> *c_data (in) 搬送波
// Buf_t<char> *s_data (in) シリアルデータ
// int       baud   (in) 600baud:0 1200baud:1
// int       frip   (in) 1->0->1->0: 0 0->1->0->1:1
// 戻り値: 次の位置
int PARSEWAV::decode_to_serial_data(Buf_t<char> *c_data, Buf_t<char> *s_data, int baud, int frip) 
{
	int pos = -1;
	int idx_ptn = (baud & 1) * IDX_PTN_1200 + frip;	// 600:0 1200:4
	int len;
	int ptn;
	char bit_data[10];
	UINT32 ms;
	const char *pptn;

	strcpy(bit_data, "");
	for(ptn=0; ptn<MAX_CARRIER_PATTERN; ptn++) {
		len = strlen(carrier_pattern[ptn][idx_ptn]);
		pptn = carrier_pattern[ptn][idx_ptn];
		if (c_data->cmp(0, pptn, len) == 0) {	// 0
			// 0
			strcpy(bit_data, "0");
			pos = len;
		}
		if (pos > 0) break;

		len = strlen(carrier_pattern[ptn][idx_ptn + 2]);
		pptn = carrier_pattern[ptn][idx_ptn + 2];
		if (c_data->cmp(0, pptn, len) == 0)	{ // 1
			// 1
			strcpy(bit_data, "1");
			pos = len;
		}
		if (pos > 0) break;
	}

	ms = c_data->get_ms(0);

	if (pos >= 0) {
		// データ追加
		s_data->adds(bit_data, ms);

		c_data->r_pos += pos;
		c_data->total_r_pos += pos;
	} else {
		// 一致するパターンがない
		strcpy(bit_data, "?");
		s_data->adds(bit_data, ms);

		c_data->r_pos += 1;
		c_data->total_r_pos += 1;

		rep_p2_error_num++;

		if (param.debug_log > 1) {
			// デバッグログ
			sprintf(cbuff,"p2 c:%12d(%s) s:%8d: error ("
				, c_data->total_r_pos
				, get_time_cstr(ms)
				, s_data->total_w_pos
				);
			strncat(cbuff, c_data->curptr(0)
				, c_data->sublen() >= len ? len : c_data->sublen());
			strcat((char *)cbuff, ")\n");
				if (fiolog) fwrite((void *)cbuff, strlen(cbuff), 1, fiolog);
		}
	}

	return pos;
}

// 搬送波からボーレートを判定する
// 引数:
// Buf_t<char> *c_data (in) 搬送波
// 戻り値: 解析した位置 / 該当なし
int PARSEWAV::parse_carrier_pattern(Buf_t<char> *c_data)
{
	int len;
	int pos = -1;
	int pos0 = 0;
	int pos1 = 0;
	int idx_ptn;
	int ptn;
	char buf0[80];
	char buf1[80];

	for(ptn=0; ptn<MAX_CARRIER_PATTERN; ptn++) {
		for(idx_ptn = 0; idx_ptn < 2; idx_ptn++) {
			// 600 or 1200ボーで0になる位置を集計
			strcpy(buf0, carrier_pattern[ptn][idx_ptn * IDX_PTN_1200]);
			strcpy(buf1, carrier_pattern[ptn][idx_ptn * IDX_PTN_1200 + 1]);
			for(int i=0; i<8; i++) {
				strcat(buf0, carrier_pattern[ptn][idx_ptn * IDX_PTN_1200 + 2]);
				strcat(buf1, carrier_pattern[ptn][idx_ptn * IDX_PTN_1200 + 3]);
			}
			pos0=c_data->find(0, buf0);
			pos1=c_data->find(0, buf1);
			if (pos0 >= 0) {
				// 順波
				st_chkwav.rev_num[st_chkwav.num][0]++;
			}
			if (pos1 >= 0) {
				// 逆波
				st_chkwav.rev_num[st_chkwav.num][1]++;
			}
			if (pos0 >= 0 || pos1 >= 0) {
				len = strlen(buf0);
				st_chkwav.baud_num[st_chkwav.num][idx_ptn]++;
				break;
			}
		}
	}
	if (pos0 >= 0 || pos1 >= 0) {
		if (pos0 >= pos1) {
			pos = pos0 + len;
		} else if (pos1 >= pos0) {
			pos = pos1 + len;
		}
	}
	return pos;
}

// ボーレートを判定してシリアルデータを変換
// 引数:
// Buf_t<char> *s_data  (in)  シリアルデータ
// Buf_t<char> *sn_data (out) 変換後シリアルデータ
// 戻り値:
int PARSEWAV::conv_baud_rate(Buf_t<char> *s_data, Buf_t<char> *sn_data)
{
	int pos = 0;
	UINT32 ms;
	UINT8 baud;
	int sn_w_pos;
	int s_r_pos;
	char buf_s[100];

	// ボーレート自動判定 
	if (param.auto_baud == 1) {
		s_r_pos = s_data->total_r_pos;
		sn_w_pos = sn_data->total_w_pos;

		if (s_data->cmp(0, "0011111111111111111111",22) == 0) {
			pos = 22;
			if (phase2n_baud_count > 0) {
				//　クリア
				for(int i=phase2n_baud_count; i>0; i--) {
					ms = s_data->get_ms(0) - (i * 110 / 600 / (1 << phase3_baud));
					baud = phase3_baud + 1;
					sn_data->addns("01111111111", ms, baud, 11);
				}
				phase2n_baud_count = 0;
			}
			phase2n_baud_count--;
			if (phase2n_baud_count < -4) {
				// 連続していたらボーレート切り替え
				phase3_baud = 0;
				for(int i=phase2n_baud_count; i<0; i++) {
					ms = s_data->get_ms(0) + (i * 110 / 600 / (1 << phase3_baud)); 
					baud = phase3_baud + 1;
					sn_data->addns("01111111111", ms, baud, 11);
				}
				phase2n_baud_count = 0;
			}
		} else if (s_data->cmp(0, "01111111111",11) == 0) {
			pos = 11;
			if (phase2n_baud_count < 0) {
				//　クリア
				for(int i=phase2n_baud_count; i<0; i++) {
					ms = s_data->get_ms(0) + (i * 110 / 600 / (1 << phase3_baud)); 
					baud = phase3_baud + 1;
					if (phase3_baud == 1) {
						sn_data->addns("0011111111111111111111", ms, baud, 22);
					} else {
						sn_data->addns("01111111111", ms, baud, 11);
					}
				}
				phase2n_baud_count = 0;
			}
			phase2n_baud_count++;
			if (phase2n_baud_count > 4) {
				// 連続していたらボーレート切り替え
				phase3_baud = 1;
				for(int i=phase2n_baud_count; i>0; i--) {
					ms = s_data->get_ms(0) - (i * 110 / 600 / (1 << phase3_baud)); 
					baud = phase3_baud + 1;
					sn_data->addns("01111111111", ms, baud, 11);
				}
				phase2n_baud_count = 0;
			}
		} else {
			if (phase2n_baud_count > 0) {
				//　クリア
				for(int i=phase2n_baud_count; i>0; i--) {
					ms = s_data->get_ms(0) - (i * 110 / 600 / (1 << phase3_baud)); 
					baud = phase3_baud + 1;
					sn_data->addns("01111111111", ms, baud, 11);
				}
			} else if (phase2n_baud_count < 0) {
				//　クリア
				for(int i=phase2n_baud_count; i<0; i++) {
					ms = s_data->get_ms(0) + (i * 110 / 600 / (1 << phase3_baud)); 
					baud = phase3_baud + 1;
					if (phase3_baud == 1) {
						sn_data->addns("0011111111111111111111", ms, baud, 22);
					} else {
						sn_data->addns("01111111111", ms, baud, 11);
					}
				}
			}
			phase2n_baud_count = 0;
		}
		// $FF以外の値
		if (pos == 0) {
			if (phase3_baud == 0) {
				// 600ボーの場合は2バイトごと
				if (s_data->get(0) != s_data->get(1)) {
					pos = 1;
				} else {
					baud = phase3_baud + 1;
					sn_data->add(s_data->get(0), s_data->get_ms(0), baud);
					pos = 2;
				}
			} else {
				baud = phase3_baud + 1;
				sn_data->add(s_data->get(0), s_data->get_ms(0), baud);
				pos = 1;
			}
		}
		s_data->r_pos+=pos;
		s_data->total_r_pos+=pos;

		if (param.debug_log > 0) {
			s_r_pos = s_data->total_r_pos - s_r_pos;
			sn_w_pos = sn_data->total_w_pos - sn_w_pos;
			memset(buf_s, 0, sizeof(buf_s));
			strncpy(buf_s, &s_data->data[s_data->r_pos - s_r_pos], s_r_pos);
			int df = strcmp(buf_s, &sn_data->data[sn_data->w_pos - sn_w_pos]);
			sprintf(cbuff, "p2n b:%d df:%2d s:%2d:%s sn:%2d:%s\n"
				,phase3_baud, df, s_r_pos
				, buf_s
				, sn_w_pos
				, &sn_data->data[sn_data->w_pos - sn_w_pos]
				);
			if (fiolog) fwrite((void *)cbuff, strlen(cbuff), 1, fiolog);
		}

	} else {
		baud = param.baud + 1;
		sn_data->add(s_data->get(0), s_data->get_ms(0), baud);
		s_data->r_pos++;
		s_data->total_r_pos++;

	}
	return pos;
}

// スタートビット（と思われる）部分を探す
// 引数:
// Buf_t<char> *s_data (in) シリアルデータ
// 戻り値: 1:スタートビット 0:?
int PARSEWAV::find_start_serial_bit(Buf_t<char> *s_data)
{
	int pos = 0;
	if ((s_data->get(0) & 1) == 0) {
		pos = 1;
	} else {
		if (param.debug_log > 1) {
			sprintf(cbuff, "p3 s:% 8d %6dms: skip [%c]\n"
				, s_data->total_r_pos
				, s_data->get_ms(0)
				, s_data->get(0));
			if (fiolog) fwrite((void *)cbuff, strlen(cbuff), 1, fiolog);
		}
		s_data->r_pos++;
		s_data->total_r_pos++;
	}

	return pos;
}

// シリアルデータをバイナリデータに変換する
// 引数:
// Buf_t<char>  *s_data (in)  シリアルデータ
// Buf_t<UINT8> *b_data (out) バイナリデータ
// 戻り: 次の位置
int PARSEWAV::decode_to_binary_data(Buf_t<char> *s_data, Buf_t<UINT8> *b_data) 
{
	int pos;
	int bin_data;
	int h_count = 0;	// H bitcount (use for parity check)
	UINT32 ms = 0;
	char *p;


	p = (char *)&(s_data->data[s_data->r_pos]); // 続きから
	bin_data = 0;
	pos = 0;
	ms = s_data->get_ms(0);
	// start bit
	pos++;
	// data bit
	for(; pos < 8 + ((param.word_select & 0x04) ? 1 : 0); pos++) {
		bin_data |= (p[pos] & 1) << (pos - 1);
		h_count += (p[pos] & 1);
	}
	// parity bit
	if ((param.word_select & 0x07) != 0x04 && (param.word_select & 0x07) != 0x05) {
		if (param.word_select & 0x01) {
			// odd parity
			if (p[pos] == (h_count & 0x01)) bin_data |= 0xff00;		// error
		} else {
			// even parity
			if (p[pos] == (1 - (h_count & 0x01))) bin_data |= 0xff00;	// error
		}
		pos++;
	}
	// stop bit
	if ((p[pos] & 1) == 0) bin_data |= 0xff00;		// error
	pos++;
	if ((param.word_select & 0x07) == 0x04 || (param.word_select & 0x06) == 0x00) {
		// 2 stop bit のエラーは無視
//			if ((p[pos] & 1) == 0) bin_data |= 0xff00;		// error
		if ((p[pos] & 1) != 0) pos++;
	}

	// データ追加
	if ((bin_data & 0xff00) == 0) {
		UINT8 attr = 0;
		if (infile_type <= 1) {
			attr = param.auto_baud ? s_data->get_attr(0) : param.baud + 1;
		}
		b_data->add(bin_data & 0xff, ms, attr);
	} else {
		// エラーの場合
		rep_p3_error_num++;
		if (param.debug_log) {
			// デバッグログ
			sprintf(cbuff,"p3 s:%8d(%s) baud error.\n"
				, s_data->total_r_pos
				, get_time_cstr(ms)
				);
			if (fiolog) fwrite((void *)cbuff, strlen(cbuff), 1, fiolog);
		}
	}
	s_data->r_pos += pos;
	s_data->total_r_pos += pos;

	return pos;
}

// バイナリデータをシリアルデータに変換する
//
//
//
//
int PARSEWAV::encode_to_serial_data(UINT8 bin_data, Buf_t<char> *s_data) 
{
	int pos = 0;
	int h_count = 0;	// H bitcount (use for parity check)
	char val;
	char *p;

	p = (char *)&(s_data->data[s_data->w_pos]);	// 続きから

	// start bit
	p[0] = '0';
	// data bit
	for(pos = 0; pos < 7 + ((param.word_select & 0x04) ? 1 : 0); pos++) {
		val = (bin_data & (1 << pos)) ? '1' : '0';
		p[pos+1] = val;
		h_count += (val == '1') ? 1 : 0;
	}
	pos++;
	// parity bit
	if ((param.word_select & 0x07) != 0x04 && (param.word_select & 0x07) != 0x05) {
		if (param.word_select & 0x01) {
			// odd parity
			val = (h_count & 0x01) ? '1' : '0';
		} else {
			// even parity
			val = (h_count & 0x01) ? '0' : '1';
		}
		p[pos] = val;
		pos++;
	}
	// stop bit
	val = '1';
	p[pos] = val;
	pos++;
	if ((param.word_select & 0x07) == 0x04 || (param.word_select & 0x06) == 0x00) {
		// 2 stop bit
		p[pos] = val;
		pos++;
	}

	return pos;
}

// シリアルデータを搬送波ビットデータに変換する
//
//
//
//
int PARSEWAV::encode_to_carrier_data(Buf_t<char> *s_data, Buf_t<char> *c_data)
{
	int idx_ptn = (param.baud & 1) * 4;	// 600:0 1200:4
	int len = 0;
	char data;
	UINT32 ms;

	s_data->get(data, ms);
	if (data & 0x01) {
		// 1
		len = c_data->adds(carrier_pattern[0][idx_ptn + 2], ms);
	} else {
		// 0
		len = c_data->adds(carrier_pattern[0][idx_ptn], ms);
	}

	return len;
}

// 搬送波ビットデータをwaveサンプルデータに変換する
// 48000Hz 16bit
//
//
//
int PARSEWAV::encode_to_wave_data(Buf_t<char> *c_data, UINT8 *w_data)
{
	int pos = 0;
	INT8 wav_samples[10];

	memset(wav_samples, 0x70, 10);

	if (c_data->r_pos <= 0
	|| (c_data->data[c_data->r_pos-1] & 1) != (c_data->data[c_data->r_pos] & 1)) {
		wav_samples[0] = 0x6c;
	}
	if (c_data->r_pos >= c_data->w_pos - 1
	|| (c_data->data[c_data->r_pos] & 1) != (c_data->data[c_data->r_pos+1] & 1)) {
		wav_samples[9] = 0x6c;
	}

	// convert to wave sample data
	for(int i=0; i<10; i++) {
		w_data[pos++]=0x80 + ((c_data->data[c_data->r_pos] & 1) ? wav_samples[i] : - wav_samples[i]);
	}
	return pos;
}


// waveサンプルデータから搬送波(1200/2400Hz)を解析し
// ビットデータに変換する。(FSK)
// 引数:
// Buf_t<UINT8> *w_data  (in) waveサンプルデータ
// Buf_t<UINT8> *wc_data (in/out) waveサンプル補正後データ
// Buf_t<char> *c_data (out) 搬送波
// Buf_t<char> *s_data  (in) シリアルデータ
// int start_phase   (in) 開始フェーズ
// 戻り値: 次の位置
int PARSEWAV::decode_phase1(Buf_t<UINT8> *w_data, Buf_t<UINT8> *wc_data, Buf_t<char> *c_data, Buf_t<char> *s_data, Buf_t<char> *sn_data, Buf_t<UINT8> *b_data, int start_phase) 
{
	return decode_phase1(w_data, wc_data, c_data, s_data, sn_data, b_data, start_phase, param.correct_type, param.reverse); 
}

int PARSEWAV::decode_phase1(Buf_t<UINT8> *w_data, Buf_t<UINT8> *wc_data, Buf_t<char> *c_data, Buf_t<char> *s_data, Buf_t<char> *sn_data, Buf_t<UINT8> *b_data, int start_phase, int correct_type, int reverse) 
{
//	int phase1h_frip = 0;
//	bool last = false;
//	bool break_data = false;
	long pos = 0;
	Buf_t<UINT8> *wn_data;

	samples = inwav.fmt.sample_rate / 1200;

	st_pa_carr.x0_prev = 0.0;
	st_pa_carr.sample_cnt = 0;
	st_pa_carr.wav_prev = 0;
	st_pa_carr.carr_prev = 0;
	st_pa_carr.odd = 0;

	st_lamda.us_delta = 1000000.0 / (double)inwav.fmt.sample_rate;
	st_lamda.us_range = 1000000.0 / 2400.0;
	st_lamda.us1200 = 1000000.0 / (double)param.freq1200 / (double)(1 << half_wave);
	st_lamda.us1800 = 1000000.0 / 1800.0 / (double)(1 << half_wave);
	st_lamda.us2400 = 1000000.0 / (double)param.freq2400 / (double)(1 << half_wave);
	st_lamda.us1200avg = st_lamda.us1200;
	st_lamda.us1800avg = st_lamda.us1800;
	st_lamda.us2400avg = st_lamda.us2400;
	if (chk_wav_mode == true) {
		st_lamda.us1200min = st_lamda.us1200 - st_lamda.us_range * 20.0 / 100.0;
		st_lamda.us1200max = st_lamda.us1200 + st_lamda.us_range * 20.0 / 100.0;
		st_lamda.us2400min = st_lamda.us2400 - st_lamda.us_range * 20.0 / 100.0;
		st_lamda.us2400max = st_lamda.us2400 + st_lamda.us_range * 20.0 / 100.0;
	} else {
		st_lamda.us1200min = st_lamda.us1200 - st_lamda.us_range * param.range1200 / 100.0;
		st_lamda.us1200max = st_lamda.us1200 + st_lamda.us_range * param.range1200 / 100.0;
		st_lamda.us2400min = st_lamda.us2400 - st_lamda.us_range * param.range2400 / 100.0;
		st_lamda.us2400max = st_lamda.us2400 + st_lamda.us_range * param.range2400 / 100.0;
	}

	if (infile_type == 0 && (param.correct == 1 || chk_wav_mode == true)) {
		wn_data = wc_data;
		init_dft(correct_type);
	} else {
		wn_data = w_data;
	}


	while(phase1 >= 0) {
		if (chk_wav_mode == true) {
			pos = (int)(sample_pos * 100 / st_chkwav.analyze_num / 2) + (int)(st_chkwav.num * 100 / 2);
		} else {
			pos = (int)(sample_pos * 100 / sample_num);
		}
		if (progbox->setProgress(pos)) {
			phase1 = -1;
			break;
		}
		switch(phase1) {
			case 0:
				// WAVファイル読み込み
				get_wav_sample(w_data, reverse);
				// チェックモードの場合は約10秒まで解析
				if (chk_wav_mode == true && sample_pos >= st_chkwav.analyze_num) {
					w_data->last = true;
				}
				phase1 = 1;
				break;
			case 1:
				// WAVサンプルデータを補正する
				if (infile_type == 0 && (param.correct == 1 || chk_wav_mode == true)) {
					calcrate_dft(w_data, wc_data);
				}
				if (outfile_type == 0 && outfile_type >= infile_type) {
					// WAV ファイル出力
					OutConvSampleData((void *)wn_data->startptr(), inwav.fmt.sample_rate, sizeof(UINT8), wn_data->length(), fio_out);
				}
				// データ最後まで読んだ場合				
				if (param.correct == 1 || chk_wav_mode == true) {
					if (w_data->last && w_data->isEOB(samples)) {
						wc_data->last = true;
					}
				}

				phase1 = 2;
				break;
			case 2:
				// L3Cへ変換
				decode_to_carrier_wave_bn(wn_data, c_data);
				if (c_data->isFull(3)) {
					// cバッファがいっぱいになったら吐き出す
					phase1 = 21;
				} else if (wn_data->last != true && wn_data->isEOB(samples)) {
					// cバッファがいっぱいになるまで繰り返す
					phase1 = 31;
				} else if (wn_data->last && wn_data->isEOB(0)) {
					// 最後のデータ
					c_data->last = true;
					if (chk_wav_mode == true) {
						phase1 = 41;
					} else {
						phase1 = 21;
					}
				}
				break;
			case 11:
				// L3Cファイル読み込み
				get_l3c_sample(c_data);
				phase1 = 21;
				break;
			case 21:
				if (outfile_type == 1 && outfile_type >= infile_type) {
					// L3C ファイル出力
					write_pos = write_data(fio_out, (char *)c_data->startptr(), 88, write_pos, 2);
				}
				// 更に変換(L3C -> L3B -> L3)
				if (outfile_type >= 0) {
					if (chk_wav_mode == true) {
						decode_phase2(c_data, s_data, sn_data, b_data, 11);
					} else {
						decode_phase2(c_data, s_data, sn_data, b_data, 1);
					}
				}
				phase1 = 31;
				break;
			case 31:
				// 残りをバッファの最初にコピー→次のターンで解析をするため
				w_data->shift();
				wc_data->shift();

				if (c_data->last && c_data->isEOB(0)) {
					phase1 = -1;
					break;
				}

				phase1 = start_phase;
				break;
			case 41:
				phase1 = -1;
				break;
		}
	}

	return phase1;
}

// 搬送波(1200/2400Hz)データからシリアルデータに変換
// ここでボーレートによってシリアルデータが変わってくる
// 引数:
// Buf_t<char> *c_data  (in) 搬送波
// Buf_t<char> *s_data  (out) シリアルデータ
// int start_phase   (in) 開始フェーズ
// 戻り値: 次の位置
int PARSEWAV::decode_phase2(Buf_t<char> *c_data, Buf_t<char> *s_data, Buf_t<char> *sn_data, Buf_t<UINT8> *b_data, int start_phase) 
{
	int step = 0;
	bool break_data = false;
	int baud = (param.auto_baud ? 1 : param.baud);
	int pos;

	while(phase2 >= 0) {
		if (chk_wav_mode == true) {
			pos = (int)(sample_pos * 100 / st_chkwav.analyze_num / 2) + (int)(st_chkwav.num * 100 / 2);
		} else {
			pos = (int)(sample_pos * 100 / sample_num);
		}
		if (progbox->setProgress(pos)) {
			phase1 = -1;
			break;
		}
		// バッファの末尾
		if (c_data->last != true && c_data->isEOB(32)) {
			break_data = true;
		}
		if (break_data == true) {
			break;
		}
		switch (phase2) {
			case 0:
				step = find_start_carrier_bit(c_data, s_data, baud, &phase2_frip);
				if (c_data->last && c_data->isEOB(0)) {
					s_data->last = true;
					phase2 = 31;
					break;
				}
				if (step >= 0) {
					if (s_data->isFull(0)) {
						phase2 = 31;
						break;
					}
					phase2 = 1;
				} else {
					break_data = true;
				}
				break;
			case 1:
				step = decode_to_serial_data(c_data, s_data, baud, phase2_frip);
				if (s_data->isFull(0)) {
					phase2 = 31;
					break;
				}
				if (c_data->isEOB(0)) {
					s_data->last = true;
					phase2 = 31;
					break;
				}
				if (step < 0) {
					phase2 = 0;
				}
				break;
			case 11:
				// 搬送波からボーレートを解析
				step = parse_carrier_pattern(c_data);
				if (step >= 0) {
					c_data->r_pos += step;
					c_data->total_r_pos += step;
				} else {
					// no data found or parsed all c_data
					c_data->total_r_pos += c_data->w_pos;
					c_data->clear();
				}
				break;
			case 21:
				get_l3b_sample(s_data);
				phase2 = 31;
				break;
			case 31:
//				if (outfile_type == 2 && outfile_type >= infile_type) {
//					write_pos = write_data(fio_out, (char *)s_data->startptr(), 110, write_pos, 1, 1);
//				}
				if (outfile_type >= 0) decode_phase2n(s_data, sn_data, b_data, 0);
				if (s_data->last && s_data->isEOB(0)) {
					phase2 = -1;
					break;
				}
				phase2 = start_phase;
				break;

		}
	}
	if (c_data->last != true && break_data == true) {
		// 残りをバッファの最初にコピー→次のターンで解析をするため
		c_data->shift();
	}
	return phase2;
}

// L3B -> ボーレート変換 ->  L3B
//
//
//
//
int PARSEWAV::decode_phase2n(Buf_t<char> *s_data, Buf_t<char> *sn_data, Buf_t<UINT8> *b_data, int start_phase) 
{
	int step = 0;
//	int baud = 0;
	bool break_data = false;

	while(phase2n >= 0) {
		if (progbox->setProgress((int)(sample_pos * 100 / sample_num))) {
			phase2n = -1;
			break;
		}
		// バッファの末尾
		if (s_data->last != true && s_data->isEOB(32)) {
			break_data = true;
		}
		if (break_data == true) {
			break;
		}
		switch (phase2n) {
			case 0:
				step = conv_baud_rate(s_data, sn_data);
				if (sn_data->isFull(131)) {
					phase2n = 31;
					break;
				}
				if (s_data->isEOB(0)) {
					sn_data->last = true;
					phase2n = 31;
					break;
				}
				break;
			case 21:
				get_l3b_sample(sn_data);
				phase2n = 31;
				break;
			case 31:
				if (outfile_type == 2 && outfile_type >= infile_type) {
					// L3Bファイル出力
					write_pos = write_data(fio_out, (char *)sn_data->startptr(), 110, write_pos, 1);
				}
				if (outfile_type >= 0) decode_phase3(sn_data, b_data, 0);
				if (s_data->last && s_data->isEOB(0)) {
					phase2n = -1;
					break;
				}
				phase2n = start_phase;
				break;

		}
	}
	if (s_data->last != true && break_data == true) {
		// 残りをバッファの最初にコピー→次のターンで解析をするため
		s_data->shift();
	}
	return phase2n;
}

// L3B -> L3
//
//
//
//
int PARSEWAV::decode_phase3(Buf_t<char> *s_data, Buf_t<UINT8> *b_data, int start_phase) 
{
	int step = 0;
	bool break_data = false;

	while(phase3 >= 0) {
		if (progbox->setProgress((int)(sample_pos * 100 / sample_num))) {
			phase3 = -1;
			break;
		}
		// バッファの末尾
		if (s_data->last != true && s_data->isEOB(32)) {
			break_data = true;
		}
		if (break_data == true) {
			break;
		}
		switch (phase3) {
			case 0:
				// find start bit
				step = find_start_serial_bit(s_data);
				if (step > 0) {
					phase3 = 1;
				}
				if (s_data->isEOB(0)) {
					b_data->last = true;
					phase3 = 41;
					break;
				}
				break;
			case 1:
				step = decode_to_binary_data(s_data, b_data);
				if (b_data->isFull(0) || s_data->isEOB(0)) {
					phase3 = 41;
					if (s_data->isEOB(0)) {
						b_data->last = true;
					}
					break;
				}
				phase3 = 0;
				break;
			case 31:
				get_l3_sample(b_data);
				phase3 = 41;
				break;
			case 41:
				if (outfile_type == 3 && outfile_type >= infile_type) {
					// L3ファイル出力
					if (b_data->length() > 0) {
						fwrite((void *)b_data->startptr(), sizeof(UINT8), b_data->length(), fio_out);
					}
				}
				if (outfile_type >= 0) decode_phase4(b_data);
				if (b_data->last && b_data->isEOB(0)) {
					phase3 = -1;
					break;
				}

				phase3 = start_phase;
				break;
		}
	}
	if (s_data->last != true && break_data == true) {
		// 残りをバッファの最初にコピー→次のターンで解析をするため
		s_data->shift();
	}
	return phase3;
}

// L3→実データ変換
//
//
//
//
int PARSEWAV::decode_phase4(Buf_t<UINT8> *b_data)
{
//	UINT8 *p = b_data->data;
	int header_type = -1;
	int data_len = 0;
	UINT8 data;
	UINT8 data_body[257];
	UINT8 *pdata;
	UINT32 ms;
	int chk_sum_calc = 0;
	int chk_sum_data = 0;
	pos_t err_pos;
	bool break_data = false;

	// ヘッダ解析
//	phase4 = 0;
	while(phase4 >= 0) {
		// バッファの末尾か？
		if (b_data->last != true && b_data->isEOB(16)) {
			break_data = true;
		} else if (b_data->last == true && b_data->isEOB(0)) {
			phase4 = -1;
			break_data = true;
		}
		if (break_data == true) {
			break;
		}

		switch(phase4) {
			case 0:
				// 0xff 0x01 0x3c を探す
				if(b_data->cmp(0, cmt_ident, 3) == 0) {
					// found
					if (rep4_itm == NULL) rep4_itm = new REPORT4(); 
//					rep_p4_header_num++;
					// type?
					header_type = b_data->get(3);
					switch(header_type) {
						case 0:
							// file name section
							phase4 = 10;
							break;
						case 1:
							// body data section
							phase4 = 11;
							break;
						case 0xff:
							// footer data section
							phase4 = 21;
							break;
						default:
							// ?? error?
							break;
					}
					b_data->r_pos += 4;
					b_data->total_r_pos += 4;
				} else {
					b_data->r_pos++;
					b_data->total_r_pos++;

				}
				break;
			case 10:
				// file name section
				rep4_itm->flags |= 1;
				rep4_itm->baud = b_data->get_attr(0);

				data_len = b_data->get(0);
				if (b_data->last != true && b_data->isEOB(data_len + 6)) {
					// バッファをフラッシュ
					break_data = true;
					break;
				}
				b_data->get(data, ms);

				chk_sum_calc = 0;

				chk_sum_calc += data_len;
				for(int i=0; i<data_len && i<20; i++) {
					b_data->get(data, ms);
					save_data_name[i] = data;
					chk_sum_calc += data;
				}
				save_data_name[20]='\0';

				memcpy(rep4_itm->save_data_name, save_data_name, 21);

				b_data->get(data, ms);
				chk_sum_data = (int)data;

				chk_sum_calc = (chk_sum_calc & 0xff);

				if (chk_sum_calc != chk_sum_data) {
					// レポート用
					err_pos.pos = b_data->total_r_pos;
					err_pos.ms  = b_data->get_ms(0);
					(rep4_itm->chksum_err_pos).push_back(err_pos);
					rep4_itm->chksum_err_num++;
				}

				// open file
				if (outfile_type == 4 && outfile_type >= infile_type) {
					if (param.file_split) {
						_stprintf(outsfile,_T("%s_%03d%s"), outsfileb, (int)rep4.size()+1, outsext);
						if (fio_outs != NULL) fclose(fio_outs);
						fio_outs = _tfopen(outsfile, _T("wb"));
					}
					prev_data_len = 0;
					memset(prev_data_body, 0, sizeof(prev_data_body));
				}

				phase4 = 0;
				break;
			case 11:
				// body data section
				rep4_itm->flags |= 2;

				data_len = b_data->get(0);
				if (b_data->last != true && b_data->isEOB(data_len + 6)) {
					// バッファをフラッシュ
					break_data = true;
					break;
				}
				b_data->get(data, ms);

				chk_sum_calc = 1;

				rep4_itm->data_count++;

				chk_sum_calc += data_len;
				for(int i=0; i<data_len && i<255; i++) {
					b_data->get(data, ms);
					data_body[i] = data;
					chk_sum_calc += data;
				}
				data_body[255]='\0';

				b_data->get(data, ms);
				chk_sum_data = (int)data;

				chk_sum_calc = (chk_sum_calc & 0xff);

				if (chk_sum_calc != chk_sum_data) {
					// レポート用
					err_pos.pos = b_data->total_r_pos;
					err_pos.ms  = ms;
					(rep4_itm->chksum_err_pos).push_back(err_pos);
					rep4_itm->chksum_err_num++;
				}

				// write to file
				if (outfile_type == 4 && outfile_type >= infile_type) {
					// マシン語のヘッダを取り除く場合
					if (param.del_mhead && rep4_itm->save_data_name[8] == 2 && rep4_itm->data_count == 1) {
						pdata = &data_body[5];
						data_len -= 5;
					} else {
						pdata = data_body;
					}

					if (prev_data_len > 0) {
						if (!param.file_split) {
							fwrite(prev_data_body, sizeof(UINT8), prev_data_len, fio_out);
						} else {
							if (fio_outs != NULL) {
								fwrite(prev_data_body, sizeof(UINT8), prev_data_len, fio_outs);
							}
						}
					}
					prev_data_len = data_len;
					memcpy(prev_data_body, pdata, data_len);
				}

				phase4 = 0;

				break;
			case 21:
				// footer data section
				rep4_itm->flags |= 4;

				data_len = b_data->get(0);
				if (b_data->last != true && b_data->isEOB(data_len + 6)) {
					// バッファをフラッシュ
					break_data = true;
					break;
				}
				b_data->get(data, ms);

				chk_sum_calc = 255;

				chk_sum_calc += data_len;
				for(int i=0; i<data_len && i<255; i++) {
					b_data->get(data, ms);
					chk_sum_calc += data;
				}
				b_data->get(data, ms);
				chk_sum_data = (int)data;

				chk_sum_calc = (chk_sum_calc & 0xff);

				if (chk_sum_calc != chk_sum_data) {
					// レポート用
					err_pos.pos = b_data->total_r_pos;
					err_pos.ms  = ms;
					(rep4_itm->chksum_err_pos).push_back(err_pos);
					rep4_itm->chksum_err_num++;
				}

				// write file
				if (outfile_type == 4 && outfile_type >= infile_type) {
					// マシン語のヘッダを取り除く場合
					if (param.del_mhead && rep4_itm->save_data_name[8] == 2) {
						prev_data_len -= 5;
					}
					if (prev_data_len > 0) {
						if (!param.file_split) {
							fwrite(prev_data_body, sizeof(UINT8), prev_data_len, fio_out);
						} else {
							if (fio_outs != NULL) {
								fwrite(prev_data_body, sizeof(UINT8), prev_data_len, fio_outs);
							}
						}
					}
					if (param.file_split) {
						// close file
						if (fio_outs != NULL) {
							fclose(fio_outs);
							fio_outs = NULL;
						}
						// 一旦バッファをフラッシュする。
						break_data = true;
					}
				}

				// add report data
				rep4.push_back(rep4_itm);
				rep4_itm = NULL;

				phase4 = 0;	// end -> parse next data 

				break;
		}
	}
	if (b_data->last != true && break_data == true) {
		// 残りをバッファの最初にコピー→次のターンで解析をするため
		b_data->shift();
	}
	if (phase4 < 0 && rep4_itm != NULL) {
		// add report data
		rep4.push_back(rep4_itm);
		rep4_itm = NULL;
	}

	return phase4;
}

// 実ファイル -> L3
//
//
//
//
int PARSEWAV::encode_phase4(Buf_t<UINT8> *b_data, Buf_t<char> *s_data, Buf_t<char> *c_data)
{
	UINT8 data;
	int hlen = (param.baud && outfile_type < 2) ? 0xc0 : 0x5a;
	int dlen = 0;
	int chk_sum = 0;

	while (phase4 >= 0) {
		switch(phase4) {
			case 0:
				chk_sum = 0;

				b_data->set(0xff, 0, hlen);
				b_data->addns(&cmt_ident[1], 0, 2);
				// header_type
				b_data->add(0, 0);
				chk_sum += 0;
				// 20バイト
				dlen = 20;
				b_data->add((UINT8)dlen, 0);
				chk_sum += dlen;
				for(int i=0; i<dlen; i++) {
					data = save_data_name[i];
					b_data->add(data, 0);
					chk_sum += data;
				}
				// chk sum
				b_data->add((chk_sum & 0xff), 0);
				// 0 x4
				b_data->set(0, 0, 4);

				phase4 = 1;
				break;

			case 1:
				// body data
				chk_sum = 0;

				b_data->set(0xff, 0, hlen);
				b_data->addns(&cmt_ident[1], 0, 2);
				// header_type
				b_data->add(1, 0);
				chk_sum += 1;
				// 255バイト最大
				if (sample_num - sample_pos >= 255) {
					dlen = 255;
				} else {
					dlen = (int)(sample_num - sample_pos);
				}
				b_data->add((UINT8)dlen, 0);
				chk_sum += dlen;
				for(int i=0; i<dlen; i++) {
					data = get_rf_sample();
					b_data->add(data, 0);
					chk_sum += data;
				}
				// chk sum
				b_data->add((chk_sum & 0xff), 0);
				// 0 x 4
				b_data->set(0, 0, 4);

				if (save_data_name[9] == 0) {
					// gap reset when it's binary save
					hlen = 10;
				}

				if (sample_pos >= sample_num) {
					phase4 = 2;
				}
				break;
			case 2:
				// footer data
				chk_sum = 0;

				b_data->set(0xff, 0, hlen);
				b_data->addns(&cmt_ident[1], 0, 2);
				// header_type
				b_data->add(0xff, 0);
				chk_sum += 0xff;
				//
				dlen = 0;
				b_data->add((UINT8)dlen, 0);
				chk_sum += dlen;
				// chk sum
				b_data->add((chk_sum & 0xff), 0);
				// 0 x 4
				b_data->set(0, 0, 4);

				phase4 = -1;
				break;
		}
		if (b_data->isFull(300) || phase4 < 0) {
			if (outfile_type == 3) {
				fwrite(b_data->data, sizeof(UINT8), (b_data->w_pos), fio_out);
			}
			phase3 = 40;
			if (outfile_type < 3) encode_phase3(b_data, s_data, c_data, 40);
			b_data->clear();
		}
	}

	return phase4;
}

// L3 -> L3B
//
//
//
//
int PARSEWAV::encode_phase3(Buf_t<UINT8> *b_data, Buf_t<char> *s_data, Buf_t<char> *c_data, int start_phase)
{
//	int pos = 0;
	int step = 0;
	bool last = false;
	UINT8 bin_data = 0;

//	pos = strlen(serial_data);

	while(phase3 >= 0) {
		if (progbox->setProgress((int)(sample_pos * 100 / sample_num))) {
			phase3 = -1;
			break;
		}
		switch (phase3) {
			case 40:
				bin_data = b_data->data[b_data->r_pos++];
				if (b_data->isEOB(0)) {
					b_data->last = true;
					last = true;
				}
				phase3 = 31;
				break;
			case 30:
				bin_data = get_l3_sample();
				if (sample_pos >= sample_num) {
					b_data->last = true;
					last = true;
				}
				phase3 = 31;
				break;
			case 31:
				step = encode_to_serial_data(bin_data, s_data);
				phase3 = start_phase;
				s_data->w_pos += step;
				if (s_data->isFull(10)) {
					phase3 = 21;
				}
				if (b_data->last) {
					s_data->last = true;
					phase3 = 21;
				}
				break;
			case 20:
				get_l3b_sample(s_data);
				phase3 = 21;
				if (s_data->last) {
					last = true;
				}
				break;
			case 21:
				if (outfile_type == 2) {
					write_pos = write_data(fio_out, (char *)s_data->data, 110, write_pos, 1);
					s_data->clear();
				}
				if (outfile_type < 2) encode_phase2(s_data, c_data, 21, last);
//				pos = strlen(serial_data);
				if (last) {
					phase3 = -1;
					break;
				}
				phase3 = start_phase;
				break;
		}	
	}
	return phase3;

}

// l3b -> l3c
// シリアルデータを搬送波ビットデータに変換する
//
//
//
int PARSEWAV::encode_phase2(Buf_t<char> *s_data, Buf_t<char> *c_data, int start_phase, bool end_data)
{
//	int pos = 0;
//	int step = 0;
//	int pos_s = 0;
//	int len = 0;
	bool last = false;
	bool break_data = false;

//	len = strlen(serial_data);
//	pos = strlen(carrier_data);
//	pos = c_data->w_pos;

	while(phase2 >= 0) {
		if (progbox->setProgress((int)(sample_pos * 100 / sample_num))) {
			phase2 = -1;
			break;
		}

		// バッファの末尾
		if (end_data != true && s_data->isEOB(32)) {
			break_data = true;
		}
		if (break_data == true) {
			break;
		}

		switch (phase2) {
			case 21:
				encode_to_carrier_data(s_data, c_data);
				phase2 = start_phase;
				if (c_data->isFull(7)) {
					phase2 = 11;
				}
				if (s_data->isEOB(0)) {
					phase2 = 11;
					last = true;
				}
				break;
			case 10:
				get_l3c_sample(c_data);
				phase2 = 11;
				if (c_data->last) {
					last = true;
				}
				break;
			case 11:
				if (outfile_type == 1) {
					write_pos = write_data(fio_out, (char *)c_data->data, 88, write_pos, 2);
					c_data->clear();
				}
				if (outfile_type < 1) encode_phase1(c_data, last);
				if (last) {
					phase2 = -1;
					break;
				}
				phase2 = start_phase;
				break;
		}	
	}
	if (end_data != true && break_data == true) {
		// 残りをバッファの最初にコピー→次のターンで解析をするため
		s_data->shift();
	}
	return phase2;

}

// L3C -> WAV
//
//
//
//
int PARSEWAV::encode_phase1(Buf_t<char> *c_data, bool end_data)
{
	int pos = 0;
	int step = 0;
	bool last = false;
	bool break_data = false;
	UINT8 wav_data[20];

	while(phase1 >= 0) {
		if (progbox->setProgress((int)(sample_pos * 100 / sample_num))) {
			phase1 = -1;
			break;
		}

		// バッファの末尾
		if (end_data != true && c_data->isEOB(32)) {
			break_data = true;
		}
		if (break_data == true) {
			break;
		}

		step = encode_to_wave_data(c_data, wav_data);
		pos += step;
		c_data->r_pos++;
		c_data->total_r_pos++;

		if (c_data->isEOB(0)) {
			last = true;
		}

		if (outfile_type == 0) {
			OutConvSampleData(wav_data,48000,1,step,fio_out);
			pos = 0;
		}

		if (last) {
			phase1 = -1;
			break;
		}

	}

	if (end_data != true && break_data == true) {
		// 残りをバッファの最初にコピー→次のターンで解析をするため
		c_data->shift();
	}
	return phase1;

}

// レポートログ出力
//
//
//
//
void PARSEWAV::reporting()
{
//	size_t len;

	_stprintf(buff, _T("----- Result Report -----"));
	write_log(buff, 1);

	// input
	_stprintf(buff, _T(" [ input ]"));
	write_log(buff, 1);
	_stprintf(buff, _T(" %s"), (const _TCHAR *)infile.fn_str());
	write_log(buff, 1);
	if (infile_type < 1) {
		_stprintf(buff, _T(" %dHz %dbit %dch"),inwav.fmt.sample_rate, inwav.fmt.sample_bits, inwav.fmt.channels);
		write_log(buff, 1);
	}

	write_log(NULL, 1);

	// output
	_stprintf(buff, _T(" [ output ]"));
	write_log(buff, 1);
	_stprintf(buff, _T(" %s"), (const _TCHAR *)outfile.fn_str());
	write_log(buff, 1);

	write_log(NULL, 1);

	// phase1 report
	if (infile_type < 1 && outfile_type >= 0) {
		_stprintf(buff, _T(" [ wav -> l3c ]"));
		write_log(buff, 1);

		_stprintf(buff, _T("  Detection: %s"),(param.half_wave ? _T("Half wave") : _T("Full wave")));
		write_log(buff, 1);
		_stprintf(buff, _T("  Range: 1200Hz: %2d%% (%6.1fHz - %6.1fHz - %6.1fHz)")
			,param.range1200,(1000000.0 /st_lamda.us1200max),(1000000.0 /st_lamda.us1200),(1000000.0 /st_lamda.us1200min));
		write_log(buff, 1);
		_stprintf(buff, _T("  Range: 2400Hz: %2d%% (%6.1fHz - %6.1fHz - %6.1fHz)")
			,param.range2400,(1000000.0 /st_lamda.us2400max),(1000000.0 /st_lamda.us2400),(1000000.0 /st_lamda.us2400min));
		write_log(buff, 1);
		_stprintf(buff, _T("  Wave Reverse: %s"),(param.reverse ? _T("on") : _T("off")));
		write_log(buff, 1);
		_stprintf(buff, _T("  Correct: %s"),(param.correct ? _T("on") : _T("off")));
		write_log(buff, 1);
		if (param.correct == 1) {
			_stprintf(buff, _T("  Correct Type: %s"),(param.correct_type ? _T("sin wave") : _T("cos wave")));
			write_log(buff, 1);
		}
		_stprintf(buff, _T("  Wave Sum:"));
		write_log(buff, 1);

		_stprintf(buff, _T("  Long(1200Hz) : %8d Cent:%6.1fHz Avg:%6.1fHz")
			, rep_p1_sample_num[0]
			, 1000000.0 / st_lamda.us1200
			, 1000000.0 / st_lamda.us1200avg);
		write_log(buff, 1);
		_stprintf(buff, _T("  Short(2400Hz): %8d Cent:%6.1fHz Avg:%6.1fHz")
			, rep_p1_sample_num[1]
			, 1000000.0 / st_lamda.us2400
			, 1000000.0 / st_lamda.us2400avg);
		write_log(buff, 1);
		_stprintf(buff, _T("  Middle       : %8d Cent:%6.1fHz Avg:%6.1fHz")
			, rep_p1_sample_num[2]
			, 1000000.0 / st_lamda.us1800
			, 1000000.0 / st_lamda.us1800avg);
		write_log(buff, 1);
		_stprintf(buff, _T("  Too Long     : %8d"),rep_p1_sample_num[3]);
		write_log(buff, 1);
		_stprintf(buff, _T("  Too Short    : %8d"),rep_p1_sample_num[4]);
		write_log(buff, 1);
		_stprintf(buff, _T("  Error        : %8d"),rep_p1_sample_num[5]);
		write_log(buff, 1);

		write_log(NULL, 1);
	}

	// phase2 report
	if (infile_type < 2 && outfile_type >= 0) {
		_stprintf(buff, _T(" [ l3c -> l3b ]"));
		write_log(buff, 1);
//		_stprintf(buff, _T(" %d Baud"),(param.baud ? 1200 : 600));
//		write_log(buff, 1);

		if (carrier_data->total_r_pos > 0) {
			_stprintf(buff, _T(" %d / %d errors. (%.2f%%)"),rep_p2_error_num,carrier_data->total_r_pos,(rep_p2_error_num * 100.0 / carrier_data->total_r_pos));
			write_log(buff, 1);
		}

		write_log(NULL, 1);
	}

	// phase3 report
	if (infile_type < 3 && outfile_type >= 0) {
		_stprintf(buff, _T(" [ l3b -> l3 ]"));
		write_log(buff, 1);

		if ((param.word_select & 0x04) == 0) {
			// 7bit
			_stprintf(buff, _T(" 7bits"));
		} else {
			// 8bit
			_stprintf(buff, _T(" 8bits"));
		}

		if (param.word_select == 4 || param.word_select == 5) {
			_tcscat(buff, _T(" NoParity"));
		} else if ((param.word_select & 0x01) == 0) {
			_tcscat(buff, _T(" EvenParity"));
		} else {
			_tcscat(buff, _T(" OddParity"));
		}
		 if ((param.word_select & 0x03) == 0) {
			_tcscat(buff, _T(" 2stopbit"));
		} else {
			_tcscat(buff, _T(" 1stopbit"));
		}
		write_log(buff, 1);

		if (serial_data->total_r_pos > 0) {
			_stprintf(buff, _T(" %d / %d errors. (%.2f%%)"),rep_p3_error_num,serial_data->total_r_pos,(rep_p3_error_num * 100.0 / serial_data->total_r_pos));
			write_log(buff, 1);
		}

		write_log(NULL, 1);
	}

	// phase4 report
	if (infile_type < 4 && outfile_type >= 0) {
		_stprintf(buff, _T(" [ l3 -> real data ]"));
		write_log(buff, 1);

		if (rep4.empty()) {
			_stprintf(buff, _T(" data cannot parse."));
			write_log(buff, 1);
		} else {
			int idx = 1;
			std::vector<REPORT4 *>::iterator itm = rep4.begin();
			while ( itm != rep4.end() ) {
				_stprintf(buff, _T("%03d:"), idx);
				write_log(buff, 0);
				if ((*itm)->flags & 1) {
					_tcscpy(buff, _T(" dataname: "));
#ifdef _UNICODE
					mbstowcs(buff, (const char *)(*itm)->save_data_name, 8);
#else
					_tcsncat(buff, (_TCHAR *)(*itm)->save_data_name, 8);
#endif
					_tcscat(buff, _T("   ["));
					switch((*itm)->save_data_name[8]) {
						case 0:
							_tcscat(buff, _T("BASIC"));
							break;
						case 1:
							_tcscat(buff, _T("DATA"));
							break;
						case 2:
							_tcscat(buff, _T("Machine"));
							break;
						default:
							_tcscat(buff, _T("?"));
							break;
					}
					switch((*itm)->save_data_name[9]) {
						case 0:
							_tcscat(buff, _T(" - Binary save"));
							break;
						case 0xff:
							_tcscat(buff, _T(" - Ascii save"));
							break;
						default:
							_tcscat(buff, _T(" ?"));
							break;
					}
					_tcscat(buff, _T("]"));
					if ((*itm)->baud == 1) {
						_tcscat(buff, _T(" ( 600baud)"));
					} else if ((*itm)->baud == 2) {
						_tcscat(buff, _T(" (1200baud)"));
					}
					write_log(buff, 1);
				} else {
					_tcscpy(buff, _T(" no filename"));
					write_log(buff, 1);
					_tcscpy(buff, _T(" header section not found."));
					write_log(buff, 1);
				}
				if (((*itm)->flags & 2) == 0) {
					_tcscpy(buff, _T(" data section not found."));
					write_log(buff, 1);
				}
				if (((*itm)->flags & 4) == 0) {
					_tcscpy(buff, _T(" footer section not found."));
					write_log(buff, 1);
				}

				if ((*itm)->chksum_err_num <= 0) {
					_stprintf(buff, _T(" check sum ok."));
				} else {
					_stprintf(buff, _T(" check sum error exists."));
				}
				write_log(buff, 1);
				for(int i=0; i<(*itm)->chksum_err_num; i++) {
					_stprintf(buff, _T(" pos: %d (%06x) %s")
						,((*itm)->chksum_err_pos)[i].pos, ((*itm)->chksum_err_pos)[i].pos
						,get_time_str(((*itm)->chksum_err_pos)[i].ms));
					write_log(buff, 1);
				}

				itm++;
				idx++;
			}
		}

		write_log(NULL, 1);
	}

	// phase3 report
	if (infile_type >= 3 && outfile_type < 3) {
		_stprintf(buff, _T(" [ l3 -> l3b ]"));
		write_log(buff, 1);

		if ((param.word_select & 0x04) == 0) {
			// 7bit
			_stprintf(buff, _T(" 7bits"));
		} else {
			// 8bit
			_stprintf(buff, _T(" 8bits"));
		}

		if (param.word_select == 4 || param.word_select == 5) {
			_tcscat(buff, _T(" NoParity"));
		} else if ((param.word_select & 0x01) == 0) {
			_tcscat(buff, _T(" EvenParity"));
		} else {
			_tcscat(buff, _T(" OddParity"));
		}
		 if ((param.word_select & 0x03) == 0) {
			_tcscat(buff, _T(" 2stopbit"));
		} else {
			_tcscat(buff, _T(" 1stopbit"));
		}
		write_log(buff, 1);

		write_log(NULL, 1);
	}

	// phase2 report
	if (infile_type >= 2 && outfile_type < 2) {
		_stprintf(buff, _T(" [ l3b -> l3c ]"));
		write_log(buff, 1);
		_stprintf(buff, _T(" %d Baud"),(param.baud ? 1200 : 600));
		write_log(buff, 1);

		write_log(NULL, 1);
	}

	// phase1 report
	if (infile_type >= 1 && outfile_type < 1) {
		_stprintf(buff, _T(" [ l3c -> wav ]"));
		write_log(buff, 1);
		_stprintf(buff, _T(" %dHz %dbit %dch"),outwav.fmt.sample_rate,outwav.fmt.sample_bits,outwav.fmt.channels);
		write_log(buff, 1);

		write_log(NULL, 1);
	}
}


// 波形解析レポートログ出力
//
//
//
//
void PARSEWAV::reporting_analyze()
{
	_stprintf(buff, _T("----- Result Report -----"));
	write_log(buff, 1);

	// input
	_stprintf(buff, _T(" [ input ]"));
	write_log(buff, 1);
	_stprintf(buff, _T(" %s"), (const _TCHAR *)infile.fn_str());
	write_log(buff, 1);
	if (infile_type < 1) {
		_stprintf(buff, _T(" %dHz %dbit %dch"), inwav.fmt.sample_rate, inwav.fmt.sample_bits, inwav.fmt.channels);
		write_log(buff, 1);
	}

	write_log(NULL, 1);

	for(int i=0; i<2; i++) {
		switch(i) {
			case 0:
				_stprintf(buff, _T("  Cos Wave:"));
				break;
			case 1:
				_stprintf(buff, _T("  Sin Wave:"));
				break;
		}
		write_log(buff, 1);
		_stprintf(buff, _T("    Long(1200Hz) : %8d Cent:%6.1fHz Avg:%6.1fHz")
			, st_chkwav.sample_num[i][0]
			, 1000000.0 / st_lamda.us1200
			, 1000000.0 / st_chkwav.us1200avg[i]);
		write_log(buff, 1);
		_stprintf(buff, _T("    Short(2400Hz): %8d Cent:%6.1fHz Avg:%6.1fHz")
			, st_chkwav.sample_num[i][1]
			, 1000000.0 / st_lamda.us2400
			, 1000000.0 / st_chkwav.us2400avg[i]);
		write_log(buff, 1);
		_stprintf(buff, _T("    Middle       : %8d"),st_chkwav.sample_num[i][2]);
		write_log(buff, 1);
		_stprintf(buff, _T("    600baud: %4d  1200baud: %4d")
			,st_chkwav.baud_num[i][0],st_chkwav.baud_num[i][1]);
		write_log(buff, 1);
		_stprintf(buff, _T("    Normal : %4d  Reverse : %4d")
			,st_chkwav.rev_num[i][0],st_chkwav.rev_num[i][1]);
		write_log(buff, 1);
	}
	write_log(NULL, 1);

	// report
	if (infile_type < 1 && outfile_type >= 0) {
		_stprintf(buff, _T(" [ result ]"));
		write_log(buff, 1);

		_stprintf(buff, _T("  Wave Reverse: %s"),(param.reverse ? _T("on") : _T("off")));
		write_log(buff, 1);
		_stprintf(buff, _T("  Correct Type: %s"),(param.correct_type ? _T("sin wave") : _T("cos wave")));
		write_log(buff, 1);
		_stprintf(buff, _T("  Avg: %4dHz  %4dHz")
			,param.freq1200,param.freq2400);
		write_log(buff, 1);

		if (param.baud) {
			_stprintf(buff, _T("  1200baud"));
		} else {
			_stprintf(buff, _T("   600baud"));
		}
		write_log(buff, 1);

		write_log(NULL, 1);
	}
}

//
//
//
//
//
void PARSEWAV::write_log(_TCHAR *buff, int crlf)
{
	if (buff) {
		if (fiolog) _fputts(buff, fiolog);
		if (logbuf) _tcsncat(logbuf, buff, logbuf_size);
	}
	for (int i=0; i<crlf; i++) {
		if (fiolog) _fputts(_T("\n"), fiolog);
#ifdef _WIN32
		if (logbuf) _tcsncat(logbuf, _T("\r\n"), logbuf_size);
#else
		if (logbuf) _tcsncat(logbuf, _T("\n"), logbuf_size);
#endif
	}
}

//
//
//
//
//
void PARSEWAV::GetFileNameBase(wxString &filename)
{
	prefix_name(infile, filename);
}

//
//
//
//
//
bool PARSEWAV::base_name(const _TCHAR *src, _TCHAR *dst) 
{
	const _TCHAR *p;
#ifdef _WIN32
	p = _tcsrchr(src, '\\');
#else
	p = _tcsrchr(src, '/');
#endif
	if (p != NULL) {
		_tcscpy(dst, p+1);
	}
	return true;
}

//
//
//
//
//
bool PARSEWAV::prefix_name(wxString &src, wxString &dst) 
{
	int p = wxNOT_FOUND;

#ifdef __WXMSW__
	if ((p = src.Find('\\', true)) != wxNOT_FOUND) {
#else
	if ((p = src.Find('/', true)) != wxNOT_FOUND) {
#endif
		dst = src.Mid(p+1);
	} else {
		dst = src;
	}
	if ((p = dst.Find('.', true)) != wxNOT_FOUND) {
		dst = dst.Left(p);
	}
	return true;
}

//
//
//
//
//
int PARSEWAV::write_data(FILE *fp, char *data, int width, int pos, int type)
{
	int len = strlen(data);
	int i = 0;
	int w = 0;
	bool last = false;
	char *p0, *p1;
	int pos0, pos1;
	char buf[DATA_BUFFER_SIZE];

	if (fp == NULL) return w;

	while (last != true) {
		w = (w == 0) ? (width - (pos % width)) : width;
		if ((i+w) > len) {
			w = len - i;
			last = true;
		}

		if (type >= 1 && last != true) {
			// 改行を入れる位置を判定する
			pos0 = -1;
			if (type >= 2) {
				p0 = strstr(&data[i+w-1],"01");
			} else {
				p0 = NULL;
			}
			if (p0 != NULL) {
				pos0 = (p0 - data);
			}
			pos1 = -1;
			p1 = strstr(&data[i+w-1],"10");
			if (p1 != NULL) {
				pos1 = (p1 - data);
			}
			if ((pos0 >= 0 && pos1 < 0) || (pos0 >= 0 && pos0 < pos1)) {
				w = pos0 - i + 1;
			} else if ((pos0 < 0 && pos1 >= 0) || (pos1 >= 0 && pos0 >= pos1)) {
				w = pos1 - i + 1;
			}
			if (w >= (width * 2)) {
				w = width;
			}
		} 
		for (int n=0; n<w; n++) {
			buf[n] = data[i + n];
		}

		fwrite(buf, sizeof(char), w, fp);
		if (last != true || (w / width) >= 1) fwrite("\r\n", sizeof(char), 2, fp);

		i += w;
	}

	return w;
}

//
//
//
//
//
bool PARSEWAV::check_extension(const _TCHAR* filename, const _TCHAR* ext)
{
	int nam_len = _tcslen(filename);
	int ext_len = _tcslen(ext);
	
	if(nam_len >= ext_len) {
#if defined(_WIN32)
		if(_tcsnicmp(&filename[nam_len - ext_len], ext, ext_len) == 0) {
#else
#ifdef _UNICODE
		if(wcsncasecmp(&filename[nam_len - ext_len], ext, ext_len) == 0) {
#else
		if(strncasecmp(&filename[nam_len - ext_len], ext, ext_len) == 0) {
#endif
#endif
			return true;
		}
	}
	return false;
}

//
//
//
//
//
_TCHAR *PARSEWAV::get_time_str(UINT32 ms)
{
	static _TCHAR str[50];
	int sec = ms / 1000;
	int msec = ms % 1000;
	int min = sec / 60;
	sec = sec % 60;

	_stprintf(str, _T("%d\'%02d\".%03d"), min, sec, msec);
	return str;
}
char *PARSEWAV::get_time_cstr(UINT32 ms)
{
	static char str[50];
	int sec = ms / 1000;
	int msec = ms % 1000;
	int min = sec / 60;
	sec = sec % 60;

	sprintf(str, "%d\'%02d\".%03d", min, sec, msec);
	return str;
}

char PARSEWAV::chr(char c)
{
	if (c < 0x20 || c > 0x7f) c = '.';
	return c;
}

//
//
//
//
//
void PARSEWAV::SetLogBufferPtr(_TCHAR *buf, long size)
{
	logbuf = buf;
	logbuf_size = size;

}

// 実ファイルのファイルフォーマット種類を得る
//
//
//
//
int  PARSEWAV::GetRfDataFormat()
{
	return rftypebox->getRfDataFormat();
}

// 実ファイルのファイルフォーマット種類ダイアログを表示
//
//
//
//
int PARSEWAV::ShowRfTypeBox()
{
	// ファイルの種類を選択
	int rc = rftypebox->showRftypeBox((_TCHAR *)NULL, false);
	if (rc != 1) {
		// cancel button
		return -1;
	}

	if (fio_in != NULL) {
		return get_first_rf_data(fio_in);
	}
	return 0;
}

// マシン語情報ダイアログを表示
//
//
//
//
bool PARSEWAV::ShowMAddressBox(bool show_cancel_button) {
	int rc;
	UINT addr;

	rc = maddressbox->showMAddressBox(show_cancel_button);
	if (rc == 1) {
		rf_header[0] = 0;
		addr = maddressbox->getStartAddr();
		rf_header[3] = addr / 256;
		rf_header[4] = addr % 256;
		addr = maddressbox->getDataSize();
		rf_header[1] = addr / 256;
		rf_header[2] = addr % 256;

		rf_footer[0] = 0xff;
		rf_footer[1] = 0;
		rf_footer[2] = 0;
		addr = maddressbox->getExecAddr();
		rf_footer[3] = addr / 256;
		rf_footer[4] = addr % 256;

		include_header = maddressbox->getIncHeader();

		return true;
	}
	return false;
}
