//
// parsewav.h
//
#ifndef _PARSEWAV_H_
#define _PARSEWAV_H_

#include <stdio.h>
#include <string.h>
#define _USE_MATH_DEFINES
#include <math.h>
#include <vector>
#include "common.h"
#include "wx/wx.h"
#include "parsewav_param.h"
#include "parsewav_buffer.h"
#include "progressbox.h"
#include "rftypebox.h"
#include "maddressbox.h"

#pragma pack(1)
typedef struct wav_header_st {
	char RIFF[4];
	UINT32 file_len;
	char WAVE[4];
} wav_header_t;
#pragma pack()

#pragma pack(1)
typedef struct wav_fmt_chank_st {
	char fmt[4];
	UINT32 fmt_size;
	UINT16 format_id;
	UINT16 channels;
	UINT32 sample_rate;
	UINT32 data_speed;
	UINT16 block_size;
	UINT16 sample_bits;
} wav_fmt_chank_t;
#pragma pack()

#pragma pack(1)
typedef struct wav_data_chank_st {
	char data[4];
	UINT32 data_len;
} wav_data_chank_t;
#pragma pack()

#pragma pack(1)
typedef struct wav_unknown_chank_st {
	char data[4];
	UINT32 len;
} wav_unknown_chank_t;
#pragma pack()

typedef struct pos_st {
	int    pos;
	UINT32 ms;
} pos_t;

// ʃ|[gpNX
class REPORT4
{
public:
	REPORT4() {
		memset(save_data_name, 0, sizeof(save_data_name));
		flags = 0;
		baud = 0;
		data_count = 0;
		chksum_err_num = 0;
	}
	~REPORT4() {}

	UINT8 save_data_name[21];
	UINT8 flags;
	UINT8 baud;
	int data_count;
	int chksum_err_num;
	std::vector<pos_t> chksum_err_pos;
};

class PARSEWAV
{
private:
	ProgressBox *progbox;
	RfTypeBox *rftypebox;
	MAddressBox *maddressbox;
	wxWindow *parent_window;

	FILE *fio_in;
	FILE *fio_out;
	FILE *fio_outs;

	FILE *fiolog;

	wxString infile;
	wxString outfile;
	_TCHAR outsfile[_MAX_PATH];
	_TCHAR outsfileb[_MAX_PATH];
	_TCHAR outsext[_MAX_PATH];
	wxString logfile;
	_TCHAR *logbuf;
	long    logbuf_size;

	int infile_type;
	int outfile_type;

	// input file info
	long long sample_pos;
	long long sample_num;
	long long sample_num_orig;
	UINT32    sample_ms;
	int  samples;

	bool include_header;
	UINT8 save_data_name[21];
	UINT8 rf_header[5];
	UINT8 rf_footer[5];


	PARSEWAV_PARAM param;

	int	phase1;
	int phase2;
	int phase2n;
	int phase3;
	int phase4;

//	int buffer_size;
	int phase2_frip;
	int phase2n_baud_count;
	int phase3_baud;

	int write_pos;
	int half_wave;

	struct parse_carrier_st {
		double x0_prev;
		int    wav_prev;
		int    sample_cnt;
		int    odd;
		int    carr_prev;
	} st_pa_carr;

	struct lamda_st {
		double us_delta;	// 1Tv̒(us)
		double us_range;
		double us1200;
		double us1200min;
		double us1200max;
		double us1200avg;
		double us1800;
		double us1800avg;
		double us2400;
		double us2400min;
		double us2400max;
		double us2400avg;
	} st_lamda;


	Buf_t<UINT8> *wave_data;
	Buf_t<UINT8> *wave_correct_data;
	Buf_t<char>  *carrier_data;
	Buf_t<char>  *serial_data;
	Buf_t<char>  *serial_new_data;
	Buf_t<UINT8> *binary_data;
	Buf_t<int>   *freq_data;

	int err_num;

	bool chk_wav_mode;
	struct chkwav_st {
		int num;
		int sample_num[4][3];
		int analyze_num;
		int baud_num[4][2];
		int rev_num[4][2];
		double us1200avg[4];
		double us2400avg[4];
	} st_chkwav;

	int rep_p1_sample_num[9];
	int rep_p2_error_num;
	int rep_p3_error_num;

	struct inwav_st {
		wav_header_t	 head;
		wav_fmt_chank_t  fmt;
		wav_data_chank_t data;
	} inwav;

	struct outwav_st {
		wav_header_t	 head;
		wav_fmt_chank_t  fmt;
		wav_data_chank_t data;
	} outwav;

	int remain_buf[100];
	int remain_buf_len;
	int divide;
	int surplus;

	double h[2][50];

	_TCHAR buff[1000];
	char  cbuff[1000];

	int prev_data_len;
	UINT8 prev_data_body[257];

	std::vector<REPORT4 *> rep4;
	REPORT4 *rep4_itm;

	int check_wav_format(FILE *fp, wav_header_t *head, wav_fmt_chank_t *fmt, wav_data_chank_t *data);
	int check_l3c_format(FILE *fp);
	int check_l3b_format(FILE *fp);
	int check_l3_format(FILE *fp);
	int check_rf_format(FILE *fp, const _TCHAR *in_file);
	int get_first_rf_data(FILE *fp);

	int get_wav_sample(int);
	int get_wav_sample(int, int);
	int get_wav_sample(Buf_t<UINT8> *w_data, int reverse);
	int get_l3c_sample(Buf_t<char> *c_data);
	int get_l3b_sample(Buf_t<char> *s_data);
	int   get_l3_sample(Buf_t<UINT8> *b_data);
	UINT8 get_l3_sample();
	UINT8 get_rf_sample();

	void init_dft(int type);
	int calcrate_dft(Buf_t<UINT8> *w_data, Buf_t<UINT8> *wc_data);
	int   decode_to_carrier_wave_bn(Buf_t<UINT8> *w_data, Buf_t<char> *c_data);

	int   find_start_carrier_bit(Buf_t<char> *c_data, Buf_t<char> *s_data, int baud, int *frip);
	int   conv_baud_rate(Buf_t<char> *s_data, Buf_t<char> *sn_data);
	int   find_start_serial_bit(Buf_t<char> *s_data);
	int   parse_carrier_pattern(Buf_t<char> *c_data);
	int   decode_to_serial_data(Buf_t<char> *c_data, Buf_t<char> *s_data, int baud, int frip); 
	int   decode_to_binary_data(Buf_t<char> *s_data, Buf_t<UINT8> *b_data); 
	int   encode_to_serial_data(UINT8 bin_data, Buf_t<char> *s_data); 
	int	  encode_to_carrier_data(Buf_t<char> *s_data, Buf_t<char> *c_data);
	int   encode_to_wave_data(Buf_t<char> *c_data, UINT8 *w_data);

	int   decode_phase1(Buf_t<UINT8> *w_data, Buf_t<UINT8> *wc_data, Buf_t<char> *c_data, Buf_t<char> *s_data, Buf_t<char> *sn_data, Buf_t<UINT8> *b_data, int start_phase, int corret_type, int reverse);
	int   decode_phase1(Buf_t<UINT8> *w_data, Buf_t<UINT8> *wc_data, Buf_t<char> *c_data, Buf_t<char> *s_data, Buf_t<char> *sn_data, Buf_t<UINT8> *b_data, int start_phase);
	int   decode_phase2(Buf_t<char> *c_data, Buf_t<char> *s_data, Buf_t<char> *sn_data, Buf_t<UINT8> *b_data, int start_phase);
	int   decode_phase2n(Buf_t<char> *s_data, Buf_t<char> *sn_data, Buf_t<UINT8> *b_data, int start_phase);
	int   decode_phase3(Buf_t<char> *s_data, Buf_t<UINT8> *b_data, int start_phase);
	int	  decode_phase4(Buf_t<UINT8> *b_data);


	int   encode_phase4(Buf_t<UINT8> *b_data, Buf_t<char> *s_data, Buf_t<char> *c_data);
	int   encode_phase3(Buf_t<UINT8> *b_data, Buf_t<char> *s_data, Buf_t<char> *c_data, int start_phase);
	int   encode_phase2(Buf_t<char> *s_data, Buf_t<char> *c_data, int start_phase, bool end_data);
	int   encode_phase1(Buf_t<char> *c_data, bool end_data);

	void  reporting();
	void  reporting_analyze();
	void  write_log(_TCHAR *, int);
	int   write_data(FILE *fp, char *data, int width, int pos, int type);

	bool check_extension(const _TCHAR* filename, const _TCHAR* ext);
	bool base_name(const _TCHAR *, _TCHAR *); 
	bool prefix_name(wxString &, wxString &);

	int get_sample_rate(int);
	int out_sample_buffer(INT16 *new_buf, size_t new_len, size_t size, FILE *fp);

	_TCHAR *get_time_str(UINT32 ms);
	char  *get_time_cstr(UINT32 ms);

	char chr(char c);

public:
	PARSEWAV(wxWindow *parent);
	PARSEWAV();
	~PARSEWAV();

	int		OpenDataFile(wxString &);
	void	CloseDataFile();
	int		CheckFileFormat(FILE *fp, const _TCHAR *in_file, int file_type);

	int		OpenOutFile(wxString &);
	void	CloseOutFile();

	int     InitFileHeader(FILE *);
	void    SetFileHeader(FILE *);
	void	OutConvSampleData(void *, UINT32, size_t, size_t, FILE *);

	int		ExportData();
	int		DecodeData();
	int		EncodeData();
	int		AnalyzeWave();

	void    GetFileNameBase(wxString &);

	wxString Errmsg();

	void SetLogBufferPtr(_TCHAR *, long);

	int  ShowRfTypeBox();
	bool ShowMAddressBox(bool);


	bool IsOpenedDataFile() { return (fio_in != NULL ? true : false); }
	int  GetDataFileType() { return infile_type; }
	int  GetRfDataFormat();

	PARSEWAV_PARAM* GetParam() { return &param; }
	void SetParam(PARSEWAV_PARAM* data) { param = *data; }

	int	GetRange1200() { return param.range1200; }
	int	GetRange2400() { return param.range2400; }
	int	GetCorrect() { return param.correct; }
	int	GetCorrectType() { return param.correct_type; }
	int	GetSampleRate() { return param.sample_rate; }
	int	GetSampleBits() { return param.sample_bits; }
	int	GetBaud() { return param.baud; }
	int	GetWordSelect() { return param.word_select; }
	int GetDebugMode() { return param.debug_log; }
	void SetRange1200(int data) { param.range1200 = data; }
	void SetRange2400(int data) { param.range2400 = data; }
	void SetCorrect(int data) { param.correct = data; }
	void SetCorrectType(int data) { param.correct_type = data; }
	void SetSampleRate(int data) { param.sample_rate = data; }
	void SetSampleBits(int data) { param.sample_bits = data; }
	void SetBaud(int data) { param.baud = data; }
	void SetWordSelect(int data) { param.word_select = data; }
	void SetDebugMode(int data) { param.debug_log = data; }

};

#endif
