//
// 変換データ保持用ストリームバッファ
//

#ifndef _PARSEWAV_BUFFER_H_
#define _PARSEWAV_BUFFER_H_

#include "common.h"

#define TERMINATOR_SIZE		4
//#ifdef _DEBUG
//#define DATA_BUFFER_SIZE	(8192 + TERMINATOR_SIZE)
//#else
#define DATA_BUFFER_SIZE	(32768 + TERMINATOR_SIZE)
//#endif

// 変換データ保持用ストリームバッファ //
template <class TYPE>
class Buf_t {
public:
	TYPE   *data;
	UINT32 *ms;
	UINT8  *attr;
	size_t  size;
	int     start_pos;
	int     w_pos;
	int     r_pos;
	int     total_w_pos;
	int     total_r_pos;
	bool    last;

	Buf_t(bool use_ms, bool use_attr, size_t init_size) {
		size = init_size - TERMINATOR_SIZE;
		ms = NULL;
		attr = NULL;
		data = new TYPE[init_size];
		if (use_ms) ms = new UINT32[init_size];
		if (use_attr) attr = new UINT8[init_size];
	}
	Buf_t(bool use_ms, bool use_attr) {
		size = DATA_BUFFER_SIZE - TERMINATOR_SIZE;
		ms = NULL;
		attr = NULL;
		data = new TYPE[DATA_BUFFER_SIZE];
		if (use_ms) ms = new UINT32[DATA_BUFFER_SIZE];
		if (use_attr) attr = new UINT8[DATA_BUFFER_SIZE];
	}
	Buf_t(bool use_ms) {
		size = DATA_BUFFER_SIZE - TERMINATOR_SIZE;
		ms = NULL;
		attr = NULL;
		data = new TYPE[DATA_BUFFER_SIZE];
		if (use_ms) ms = new UINT32[DATA_BUFFER_SIZE];
	}
	Buf_t() {
		size = DATA_BUFFER_SIZE - TERMINATOR_SIZE;
		ms = NULL;
		attr = NULL;
		data = new TYPE[DATA_BUFFER_SIZE];
	}
	~Buf_t() {
		if (attr) delete [] attr;
		if (ms) delete [] ms;
		delete [] data;
	}
	void init(void) {
		clear();
		total_w_pos = 0;
		total_r_pos = 0;
		last = false;
	}
	void clear(void) {
		for(size_t i=0; i<(size + TERMINATOR_SIZE); i++) {
			data[i] = 0;
			if (ms) ms[i] = 0;
			if (attr) attr[i] = 0;
		}
		start_pos = 0;
		w_pos = 0;
		r_pos = 0;
	}
	// バッファにデータを追加 //
	void add(TYPE c) {
		add(c, 0, 0);
	}
	void add(TYPE c, UINT32 ss) {
		add(c, ss, 0);
	}
	void add(TYPE c, UINT32 ss, UINT8 att) {
		if (w_pos >= (int)size) return;
		data[w_pos] = c;
		if (ms) ms[w_pos] = ss;
		if (attr) attr[w_pos] = att;
		w_pos++;
		total_w_pos++;
	}
	// バッファにデータを追加 //
	int adds(const TYPE *dat) {
		return adds(dat, 0, 0);
	}
	int adds(const TYPE *dat, UINT32 ss) {
		return adds(dat, ss, 0);
	}
	int adds(const TYPE *dat, UINT32 ss, UINT8 att) {
		int len = 0;
		while(*dat != 0 && w_pos<(int)size) {
			add(*dat, ss, att);
			dat++;
			len++;
		}
		return len;
	}
	// バッファにデータを追加 //
	int addns(const TYPE *dat, size_t dat_len) {
		return addns(dat, 0, 0, dat_len);
	}
	int addns(const TYPE *dat, UINT32 ss, size_t dat_len) {
		return addns(dat, ss, 0, dat_len);
	}
	int addns(const TYPE *dat, UINT32 ss, UINT8 att, size_t dat_len) {
		int len = 0;
		for(size_t i=0; i<dat_len && w_pos<(int)size; i++) {
			add(dat[i], ss, att);
			len++;
		}
		return len;
	}
	// バッファに同じデータを追加 //
	int set(TYPE c, UINT32 ss, size_t dat_len) {
		int len = 0;
		for(size_t i=0; i<dat_len && w_pos<(int)size; i++) {
			add(c, ss);
			len++;
		}
		return len;
	}
	// データを得る //
	UINT32 get_ms(int offset) {
		if (ms) {
			return ms[r_pos + offset];
		} else {
			return 0;
		}
	}
	UINT8 get_attr(int offset) {
		if (attr) {
			return attr[r_pos + offset];
		} else {
			return 0;
		}
	}
	TYPE get(int offset) {
		return data[r_pos + offset];
	}
	void get(TYPE &c, UINT32 &ss) {
		if (r_pos >= w_pos) {
			c = get(-1);
			ss = get_ms(-1);
		} else {
			c = get(0);
			ss = get_ms(0); 
			r_pos++;
			total_r_pos++;
		}
	}
	// 未処理のデータをバッファの先頭にシフトする //
	void shift() {
		if (r_pos == 0) return;
		for(int i=0; i<(w_pos - r_pos); i++) {
			data[i] = data[i+r_pos];
			if (ms) ms[i] = ms[i+r_pos];
			if (attr) attr[i] = attr[i+r_pos];
		}
		w_pos = w_pos - r_pos;
		for(int i=w_pos; i<(int)(size + TERMINATOR_SIZE); i++) {
			data[i] = 0;
			if (ms) ms[i] = 0;
			if (attr) attr[i] = 0;
		}
		r_pos = 0;
		start_pos = w_pos;
	}
	TYPE *startptr(void) {
		return &data[start_pos];
	}
	int length(void) {
		return w_pos - start_pos;
	}
	TYPE *curptr(int offset) {
		return &data[r_pos+offset];
	}
	int sublen(void) {
		return w_pos - r_pos;
	}
	bool isFull(int offset) {
		return (w_pos + offset >= (int)size) ? true : false;
	}
	bool isEOB(int offset) {
		return (r_pos + offset >= w_pos) ? true : false;
	}
	// 比較 //
	int cmp(int offset, const TYPE *dat, size_t len) {
		return memcmp((void *)&data[r_pos + offset], (void *)dat, len * sizeof(TYPE));
	}
	// 探す charのみ //
	int find(int offset, const char *str) {
		int idx = -1;
		char *p = strstr(&data[r_pos + offset], str);
		if (p != NULL) {
			idx = p - &data[r_pos + offset];
		}
		return idx; 
	}
};

#endif
