﻿//
// 実ファイル種類設定ダイアログ
//
#include "rftypebox.h"

// Attach Event
BEGIN_EVENT_TABLE(RfTypeBox, wxDialog)
	EVT_RADIOBUTTON(IDC_RF_BASIC, RfTypeBox::OnClickBasic)
	EVT_RADIOBUTTON(IDC_RF_DATA, RfTypeBox::OnClickData)
	EVT_RADIOBUTTON(IDC_RF_MACHINE, RfTypeBox::OnClickMachine)
	EVT_BUTTON(wxID_OK, RfTypeBox::OnClickOk)
END_EVENT_TABLE()

RfTypeBox::RfTypeBox()
{
	RfTypeBox(NULL, wxID_ANY);
}

RfTypeBox::RfTypeBox(wxWindow* parent, wxWindowID id)
	: wxDialog(parent, id, _("ファイル種類"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE)
{
	wxSizerFlags flags = wxSizerFlags().Expand().Border(wxALL, 4);

	wxBoxSizer  *szrAll = new wxBoxSizer(wxVERTICAL);
	wxBoxSizer *szr;

	szrAll->Add(new wxStaticText(this, wxID_ANY, _("ファイルの種類を選択してください。")), flags);

	szr = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer *szrL = new wxStaticBoxSizer(new wxStaticBox(this, wxID_ANY, _("")), wxVERTICAL);
	radBasic = new wxRadioButton(this, IDC_RF_BASIC, _("BASIC"),wxDefaultPosition,wxDefaultSize,wxRB_GROUP);
	radData = new wxRadioButton(this, IDC_RF_DATA, _("データ"));
	radMachine = new wxRadioButton(this, IDC_RF_MACHINE, _("マシン語"));
    szrL->Add(radBasic, flags);
    szrL->Add(radData, flags);
    szrL->Add(radMachine, flags);
	szr->Add(szrL, flags);
	wxBoxSizer *szrR = new wxStaticBoxSizer(new wxStaticBox(this, wxID_ANY, _("")), wxVERTICAL);
	radBinary = new wxRadioButton(this, IDC_RF_BINARY, _("Binary"),wxDefaultPosition,wxDefaultSize,wxRB_GROUP);
	radAscii = new wxRadioButton(this, IDC_RF_ASCII, _("Ascii"));
    szrR->Add(radBinary, flags);
    szrR->Add(radAscii, flags);
	szr->Add(szrR, flags);
	szrAll->Add(szr, flags);

	szrAll->Add(new wxStaticText(this, wxID_ANY, _("内部ファイル名を入力してください。")), flags);
	txtEdit = new wxTextCtrl(this, IDC_RF_EDIT, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER);
	szrAll->Add(txtEdit, flags);

	wxSizer *szrButtons = CreateButtonSizer(wxOK|wxCANCEL);
	szrAll->Add(szrButtons, flags);

	SetSizerAndFit(szrAll);

	// initialize
	data_format = 0;
	data_type   = 0;

	rf_name = _("");
}



int RfTypeBox::getRfDataName(UINT8 *name)
{
#ifdef _UNICODE
	wcstombs((char *)name, rf_name.fn_str(), 8);
#else
	strncpy((char *)name, rf_name.fn_str(), 8);
#endif
	for(int i=0; i<8; i++) {
		if (name[i] == 0) name[i] = 0x20;
	}
	return 0;
}

// ダイアログ表示
int RfTypeBox::showRftypeBox(_TCHAR *name, bool init_data)
{
	wxString sName = _("");
	
	if (name != NULL) {
		sName = wxString(name, wxConvUTF8);
	}
	return showRftypeBox(sName, init_data);
}

int RfTypeBox::showRftypeBox(wxString &name, bool init_data)
{
	if (name.Length() > 0) {
		if (chk_str(name)) {
			rf_name = name.Left(8);
		} else {
			rf_name = _("");
		}
	}

	init_dialog(init_data);

	wxWindow *cancelBtn = FindWindowById(wxID_CANCEL, this);
	if (cancelBtn != NULL) {
		if (init_data) {
			cancelBtn->Enable(false);
		} else {
			cancelBtn->Enable(true);
		}
	}

	int rc = wxDialog::ShowModal();
	if (rc == wxID_OK) {
		term_dialog();
		return 1;
	}
	return 0;
}

void RfTypeBox::init_dialog(bool init_data)
{
	if (init_data) {
		data_format = 0;
		data_type = 0;
	}
	radBasic->SetValue(data_format == 0);
	radData->SetValue(data_format == 1);
	radMachine->SetValue(data_format == 2);
	radBinary->SetValue(data_type == 0);
	radAscii->SetValue(data_type == 0xff);
	radBinary->Enable(data_format == 2 || data_format == 0);
	radAscii->Enable(data_format == 1 || data_format == 0);

	txtEdit->SetValue(rf_name);
	txtEdit->SetMaxLength(8);

}

void RfTypeBox::term_dialog()
{
	// ラジオボタン
	if (radBasic->GetValue()) {
		data_format = 0;
	} else if (radData->GetValue()) {
		data_format = 1;
	} else {
		data_format = 2;
	}
	if (radBinary->GetValue()) {
		data_type = 0;
	} else {
		data_type = 0xff;
	}
}

void RfTypeBox::OnClickBasic(wxCommandEvent& event)
{
	radBinary->Enable(true);
	radAscii->Enable(true);
}

void RfTypeBox::OnClickData(wxCommandEvent& event)
{
	radBinary->Enable(false);
	radAscii->Enable(true);
	radBinary->SetValue(false);
	radAscii->SetValue(true);
}

void RfTypeBox::OnClickMachine(wxCommandEvent& event)
{
	radBinary->Enable(true);
	radAscii->Enable(false);
	radBinary->SetValue(true);
	radAscii->SetValue(false);
}

void RfTypeBox::OnClickOk(wxCommandEvent& event)
{
	// テキスト
	rf_name = txtEdit->GetValue();
	if (!chk_str(rf_name)) {
		wxMessageBox(_("内部ファイル名は半角英数字で入力してください。")
			, _("error"), wxOK | wxICON_ERROR, this);
		return;
	}
	EndModal(wxID_OK);
}

bool RfTypeBox::chk_str(wxString &str)
{
	bool rc = true;
	UINT len;

	len = str.Length();

	if (len <= 0) {
			rc = false;
	}
	for(UINT i=0; i < len; i++) {
		wxChar c = str.GetChar(i);
		if (!_istprint(c)
			|| c == _T('(') || c == _T(')') || c == _T(':')) {
			rc = false;
			break;
		}
	}
	return rc;
}


