﻿//
// wavtool.cpp
//
#include "parsewav.h"
#include "wavtool.h"
#include "configbox.h"
#include "wx/filename.h"
#include "wx/fileconf.h"
#include "res/wavtool.xpm"

IMPLEMENT_APP(WavtoolApp)

bool WavtoolApp::OnInit()
{
	if (!wxApp::OnInit()) {
		return false;
	}
	SetAppPath();
	SetAppName(_("wavtool"));

	WavtoolFrame *frame = new WavtoolFrame(GetAppName(), wxSize(480, 400) );
	frame->Show(true);
	SetTopWindow(frame);
	return true;
}

int WavtoolApp::OnExit()
{
	return 0;
}

void WavtoolApp::SetAppPath()
{
	wxChar *app = argv[0];

	app_path = wxFileName::FileName(wxString((const wxChar *)app, wxConvFile)).GetPath(wxPATH_GET_SEPARATOR);
}

wxString WavtoolApp::GetAppPath()
{
	return app_path;
}

//
// Frame
//
// Attach Event
BEGIN_EVENT_TABLE(WavtoolFrame, wxFrame)
	// menu event
	EVT_MENU(wxID_EXIT,  WavtoolFrame::OnQuit)
	EVT_MENU(IDM_ABOUT, WavtoolFrame::OnAbout)

	EVT_MENU(IDM_OPEN_FILE, WavtoolFrame::OnOpenFile)
	EVT_MENU(IDM_CLOSE_FILE, WavtoolFrame::OnCloseFile)

	EVT_MENU(IDM_EXPORT_REAL, WavtoolFrame::OnExportFile)
	EVT_MENU(IDM_EXPORT_L3,   WavtoolFrame::OnExportFile)
	EVT_MENU(IDM_EXPORT_L3B,  WavtoolFrame::OnExportFile)
	EVT_MENU(IDM_EXPORT_L3C,  WavtoolFrame::OnExportFile)
	EVT_MENU(IDM_EXPORT_WAV,  WavtoolFrame::OnExportFile)

	EVT_MENU(IDM_ANALYZE_WAV, WavtoolFrame::OnAnalyzeWave)

	EVT_MENU(IDM_OPTION_RFTYPE,  WavtoolFrame::OnOptionRftype)
	EVT_MENU(IDM_OPTION_MACHINE, WavtoolFrame::OnOptionMachine)

	EVT_MENU(IDM_CONFIGURE,  WavtoolFrame::OnConfigure)

	EVT_MENU_OPEN(WavtoolFrame::OnMenuOpen)

//	EVT_DROP_FILES(WavtoolFrame::OnDropFiles)
END_EVENT_TABLE()

WavtoolFrame::WavtoolFrame(const wxString& title, const wxSize& size)
       : wxFrame(NULL, -1, title, wxDefaultPosition, size)
{
	// icon
#ifdef __WXMSW__
	SetIcon(wxIcon(_("wavtool")));
#elif defined(__WXGTK__) || defined(__WXMOTIF__)
	SetIcon(wxIcon(wavtool_xpm));
#endif

	// menu
	menuFile = new wxMenu;
	menuOther = new wxMenu;
	menuHelp = new wxMenu;
	wxMenu *smenu;

	menuFile->Append( IDM_OPEN_FILE, _("開く...") );
	menuFile->Append( IDM_CLOSE_FILE, _("閉じる") );
	menuFile->AppendSeparator();
	smenu = new wxMenu;
	smenu->Append( IDM_EXPORT_REAL, _("実ファイル...") );
	smenu->AppendSeparator();
	smenu->Append( IDM_EXPORT_L3, _("L3ファイル...") );
	smenu->Append( IDM_EXPORT_L3B, _("L3Bファイル...") );
	smenu->Append( IDM_EXPORT_L3C, _("L3Cファイル...") );
	smenu->Append( IDM_EXPORT_WAV, _("WAVファイル...") );
	menuFile->Append( IDM_EXPORT, _("エクスポート"), smenu );
	menuFile->AppendSeparator();
	menuFile->Append( IDM_ANALYZE_WAV, _("波形解析") );
	menuFile->AppendSeparator();
	smenu = new wxMenu;
	smenu->Append( IDM_OPTION_RFTYPE, _("データ種類...") );
	smenu->Append( IDM_OPTION_MACHINE, _("マシン語...") );
	menuFile->Append( IDM_FILE_OPTION, _("オプション"), smenu );
	menuFile->AppendSeparator();
	menuFile->Append( wxID_EXIT, _("アプリケーションの終了") );
	// other menu
	menuOther->Append( IDM_CONFIGURE, _("設定...") );
	// help menu
	menuHelp->Append( IDM_ABOUT, _("バージョン情報...") );

	// menu bar
	wxMenuBar *menuBar = new wxMenuBar;
	menuBar->Append( menuFile, _("ファイル") );
	menuBar->Append( menuOther, _("その他") );
	menuBar->Append( menuHelp, _("ヘルプ") );

	SetMenuBar( menuBar );

	// control panel
	panel = new WavtoolPanel(this);

	//
//	DragAcceptFiles(true);

	// initialize
	wav = new PARSEWAV(this);
	wav->SetLogBufferPtr(text_buffer, MAX_BUFFER);

	cfgbox = new ConfigBox(this, IDD_CONFIGBOX);

	wav_file_path = _("");

	// load ini file
	wxString ini_file = wxGetApp().GetAppPath() + wxGetApp().GetAppName() + ".ini";
	wxFileConfig *ini = new wxFileConfig(wxEmptyString,wxEmptyString,ini_file,wxEmptyString
		,wxCONFIG_USE_LOCAL_FILE | wxCONFIG_USE_RELATIVE_PATH | wxCONFIG_USE_NO_ESCAPE_CHARACTERS);

	int lv;
	ini->Read(_("Path"), &wav_file_path);
	if (!ini->Read(_("SampleRate"), &lv)) lv = wav->GetSampleRate();
	if (lv < 0 || 3 < lv) lv = wav->GetSampleRate();
	wav->SetSampleRate(lv);
	if (!ini->Read(_("SampleBits"), &lv)) lv = wav->GetSampleBits();
	if (lv < 0 || 1 < lv) lv = wav->GetSampleBits();
	wav->SetSampleBits(lv);
	if (!ini->Read(_("Baud"), &lv)) lv = wav->GetBaud();
	if (lv < 0 || 1 < lv) lv = wav->GetBaud();
	wav->SetBaud(lv);

	delete ini;
}

WavtoolFrame::~WavtoolFrame()
{
	// save ini file
	wxString ini_file = wxGetApp().GetAppPath() + wxGetApp().GetAppName() + ".ini";
	wxFileConfig *ini = new wxFileConfig(wxEmptyString,wxEmptyString,ini_file,wxEmptyString
		,wxCONFIG_USE_LOCAL_FILE | wxCONFIG_USE_RELATIVE_PATH | wxCONFIG_USE_NO_ESCAPE_CHARACTERS);
	ini->Write(_("Path"), wav_file_path);
	ini->Write(_("SampleRate"), wav->GetSampleRate());
	ini->Write(_("SampleBits"), wav->GetSampleBits());
	ini->Write(_("Baud"), wav->GetBaud());
	// write
	delete ini;

	delete cfgbox;
	delete wav;
}

// メニュー更新
void WavtoolFrame::OnMenuOpen(wxMenuEvent& event)
{
	wxMenu *menu = event.GetMenu();

	if (menu == NULL) return;

	if (menu == menuFile) {	// File...
		// Export to ...
		menu->Enable(IDM_CLOSE_FILE,  false);
		menu->Enable(IDM_EXPORT_L3,   false);
		menu->Enable(IDM_EXPORT_L3B,  false);
		menu->Enable(IDM_EXPORT_L3C,  false);
		menu->Enable(IDM_EXPORT_WAV,  false);
		menu->Enable(IDM_EXPORT_REAL, false);
		menu->Enable(IDM_ANALYZE_WAV, false);
		// Options ...
		menu->Enable(IDM_OPTION_RFTYPE,  false);
		menu->Enable(IDM_OPTION_MACHINE, false);

		if (wav->IsOpenedDataFile()) {
			menu->Enable(IDM_CLOSE_FILE, true);

			switch(wav->GetDataFileType()) {
				case 0:	//wav
					menu->Enable(IDM_EXPORT_L3,   true);
					menu->Enable(IDM_EXPORT_L3B,  true);
					menu->Enable(IDM_EXPORT_L3C,  true);
					menu->Enable(IDM_EXPORT_WAV,  true);
					menu->Enable(IDM_EXPORT_REAL, true);
					menu->Enable(IDM_ANALYZE_WAV, true);
					break;
				case 1:	//l3c
					menu->Enable(IDM_EXPORT_L3,   true);
					menu->Enable(IDM_EXPORT_L3B,  true);
					menu->Enable(IDM_EXPORT_WAV,  true);
					menu->Enable(IDM_EXPORT_REAL, true);
					break;
				case 2:	//l3b
					menu->Enable(IDM_EXPORT_L3,   true);
					menu->Enable(IDM_EXPORT_L3C,  true);
					menu->Enable(IDM_EXPORT_WAV,  true);
					menu->Enable(IDM_EXPORT_REAL, true);
					break;
				case 3:	//l3
					menu->Enable(IDM_EXPORT_L3B,  true);
					menu->Enable(IDM_EXPORT_L3C,  true);
					menu->Enable(IDM_EXPORT_WAV,  true);
					menu->Enable(IDM_EXPORT_REAL, true);
					break;
				case 4:	//real
					menu->Enable(IDM_EXPORT_L3,   true);
					menu->Enable(IDM_EXPORT_L3B,  true);
					menu->Enable(IDM_EXPORT_L3C,  true);
					menu->Enable(IDM_EXPORT_WAV,  true);
					menu->Enable(IDM_OPTION_RFTYPE, true);
					if (wav->GetRfDataFormat() == 2) {
						menu->Enable(IDM_OPTION_MACHINE, true);
					}
					break;
				default:
					break;
			}
		}
	}
}

// ドロップされたファイルを開く
#if 0
void WavtoolFrame::OnDropFiles(wxDropFilesEvent& event)
{
	wxString *names = event.GetFiles();
	int num = event.GetNumberOfFiles();

	// 最初のファイル名を取得
	if (num > 0) {
		CloseDataFile();
		OpenDataFile(names[0]);
	}
}
#endif
void WavtoolFrame::OpenDroppedFile(wxString &path)
{
	CloseDataFile();
	OpenDataFile(path);
}

void WavtoolFrame::OnQuit(wxCommandEvent& WXUNUSED(event))
{
 	CloseDataFile();
	Close(true);
}

void WavtoolFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
{
	WavtoolAbout(this, wxID_ANY).ShowModal();
}

void WavtoolFrame::OnOpenFile(wxCommandEvent& WXUNUSED(event))
{
	WavtoolFileDialog *dlg = new WavtoolFileDialog(
		_("ファイルを開く"),
		wav_file_path,
		wxEmptyString,
		_("Supported Files (*.wav;*.l3c;*.l3b;*.l3;*.bin;*.bas;*.dat)|*.wav;*.l3c;*.l3b;*.l3;*.bin;*.bas;*.dat|All Files (*.*)|*.*"),
		wxFD_OPEN);

	int rc = dlg->ShowModal();
	wxString path = dlg->GetPath();

	delete dlg;

	if (rc == wxID_OK) {
		OpenDataFile(path);
	}
}

void WavtoolFrame::OnCloseFile(wxCommandEvent& WXUNUSED(event))
{
	CloseDataFile();
}

// エクスポート
void WavtoolFrame::OnExportFile(wxCommandEvent& event)
{
	int id = event.GetId();

	ExportFile(id);
}

// 波形解析
void WavtoolFrame::OnAnalyzeWave(wxCommandEvent& event)
{
	int rc;

	rc = wav->AnalyzeWave();

	// disp infomation on the window
	wxString sInfo(text_buffer, MAX_BUFFER);
	panel->GetTextInfo()->SetValue(sInfo);
}

// データ種類設定ダイアログ
void WavtoolFrame::OnOptionRftype(wxCommandEvent& WXUNUSED(event))
{
	int rc;

	rc = wav->ShowRfTypeBox();
	if (rc == -1) {
		// cancel button
		return;
	} else if (rc == 1) {
		wxMessageBox(wav->Errmsg(), _("Error"), wxOK | wxICON_ERROR, this);
		return;
	} else if (rc == 2) {
		wxMessageBox(wav->Errmsg(), _("Warning"), wxOK | wxICON_WARNING, this);
	}
}

// マシン語設定ダイアログ
void WavtoolFrame::OnOptionMachine(wxCommandEvent& WXUNUSED(event))
{
	wav->ShowMAddressBox(true);
}

// 設定ダイアログ
void WavtoolFrame::OnConfigure(wxCommandEvent& WXUNUSED(event))
{
	cfgbox->SetParam(wav->GetParam());

	cfgbox->ShowModal();

	wav->SetParam(cfgbox->GetParam());
}

// 指定したファイルを開く
void WavtoolFrame::OpenDataFile(wxString &path)
{
	int rc;

	rc = wav->OpenDataFile(path);
	if (rc == -1) {
		// cancel button
		return;
	} else if (rc == 1) {
		wxMessageBox(wav->Errmsg(), _("Error"), wxOK | wxICON_ERROR, this);
		return;
	} else if (rc == 2) {
		wxMessageBox(wav->Errmsg(), _("Warning"), wxOK | wxICON_WARNING, this);
	}

	wav_file_path = wxFileName::FileName(path).GetPath(wxPATH_GET_SEPARATOR);

	// update window
	wxString title = wxGetApp().GetAppName() + _(" - ") + path;
	SetTitle(title);
	panel->GetTextName()->SetValue(path);
}

// ファイルを閉じる
void WavtoolFrame::CloseDataFile()
{
	wav->CloseDataFile();

	// update window
	wxString title = wxGetApp().GetAppName();
	SetTitle(title);
	panel->GetTextName()->SetValue(_(""));
}

// エクスポート
void WavtoolFrame::ExportFile(int id)
{
	int rc;
	wxString file_base;
	wxString wild_card;

	wav->GetFileNameBase(file_base);

	switch(id) {
		case IDM_EXPORT_L3:
			file_base += _(".l3");
			wild_card = _("L3 File (*.l3)|*.l3");
			break;
		case IDM_EXPORT_L3B:
			file_base += _(".l3b");
			wild_card = _("L3B File (*.l3b)|*.l3b");
			break;
		case IDM_EXPORT_L3C:
			file_base += _(".l3c");
			wild_card = _("L3C File (*.l3c)|*.l3c");
			break;
		case IDM_EXPORT_WAV:
			file_base += _(".wav");
			wild_card = _("Wave File (*.wav)|*.wav");
			break;
		case IDM_EXPORT_REAL:
#if defined(__WXGTK__) || defined(__WXMOTIF__)
			file_base += _(".bin");
			wild_card = _("Real File (*.bin)|*.bin|All Files (*.*)|*.*");
#else
			file_base += _(".bin");
			wild_card = _("Real File (*.bin)|*.bin|Basic File (*.bas)|*.bas|Data File (*.dat)|*.dat|All Files (*.*)|*.*");
#endif
			break;
		default:
			file_base += _("");
			wild_card = _("All Files (*.*)|*.*");
			break;
	}

	WavtoolFileDialog *dlg = new WavtoolFileDialog(
		_("エクスポート"),
		wav_file_path,
		file_base,
		wild_card,
		wxFD_SAVE | wxFD_OVERWRITE_PROMPT);

	rc = dlg->ShowModal();
	wxString path = dlg->GetPath();

	delete dlg;

	if (rc == wxID_OK) {
		rc = wav->OpenOutFile(path);
		if (rc == -1) {
			// cancel button
			return;
		} else if (rc == 1) {
			wxMessageBox(wav->Errmsg(), _("Error"), wxOK | wxICON_ERROR, this);
			return;
		} else if (rc == 2) {
			wxMessageBox(wav->Errmsg(), _("Warning"), wxOK | wxICON_WARNING, this);
		}
		//
		rc = wav->ExportData();
		wav->CloseOutFile();
		if (rc == -1) {
			// cancel button
			return;
		} else if (rc == 1) {
			wxMessageBox(wav->Errmsg(), _("Error"), wxOK | wxICON_ERROR, this);
			return;
		} else if (rc == 2) {
			wxMessageBox(wav->Errmsg(), _("Warning"), wxOK | wxICON_WARNING, this);
		}

		// disp infomation on the window
		wxString sInfo(text_buffer, MAX_BUFFER);
		panel->GetTextInfo()->SetValue(sInfo);
	}
}

//
// Control Panel
//
// Attach Event
BEGIN_EVENT_TABLE(WavtoolPanel, wxPanel)
	// event
	EVT_SIZE(WavtoolPanel::OnSize)
END_EVENT_TABLE()

WavtoolPanel::WavtoolPanel(WavtoolFrame *parent)
       : wxPanel(parent)
{
	wxPoint pos;
	wxSize  frame_size = parent->GetSize();
	wxSize  size;
	long style;

	pos.x = 0; pos.y = 0;
	size.x = frame_size.x; size.y = -1;
	style = wxTE_READONLY;
	textName = new wxTextCtrl(this, IDC_TEXT_NAME, wxEmptyString, pos, size, style);
	wxSize      sizeName = textName->GetSize();

	pos.x = 0; pos.y = sizeName.y;
	size.x = frame_size.x; size.y = frame_size.y - sizeName.y;
	style = wxTE_READONLY | wxTE_MULTILINE;
	textInfo = new wxTextCtrl(this, IDC_TEXT_INFO, wxEmptyString, pos, size, style);

	// drag and drop
	textName->SetDropTarget(new WavtoolFileDropTarget(parent));
	textInfo->SetDropTarget(new WavtoolFileDropTarget(parent));
}

void WavtoolPanel::OnSize(wxSizeEvent& event)
{
	wxSize size = event.GetSize();

	textName->SetSize(size.x, textName->GetSize().y);
	textInfo->SetSize(size.x, size.y - textName->GetSize().y);
}

//
// File Dialog
//
WavtoolFileDialog::WavtoolFileDialog(const wxString& message, const wxString& defaultDir, const wxString& defaultFile, const wxString& wildcard, long style) 
            : wxFileDialog(NULL, message, defaultDir, defaultFile, wildcard, style)
{
}

//
// File Drag and Drop
//
WavtoolFileDropTarget::WavtoolFileDropTarget(WavtoolFrame *parent)
			: frame(parent)
{
}

bool WavtoolFileDropTarget::OnDropFiles(wxCoord x, wxCoord y ,const wxArrayString &filenames) {
	if (filenames.Count() > 0) {
		wxString name = filenames.Item(0);
		frame->OpenDroppedFile(name);
	}
    return true;
}

//
// About dialog
//
WavtoolAbout::WavtoolAbout(wxWindow* parent, wxWindowID id)
	: wxDialog(parent, id, _("About..."), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE)
{
	wxSizerFlags flags = wxSizerFlags().Expand().Border(wxALL, 4);

	wxBoxSizer *szrLeft   = new wxBoxSizer(wxVERTICAL);
	wxBoxSizer *szrRight  = new wxBoxSizer(wxVERTICAL);
	wxBoxSizer *szrMain   = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer *szrAll    = new wxBoxSizer(wxVERTICAL);

	szrLeft->Add(new wxStaticBitmap(this, wxID_ANY,
		wxBitmap(wavtool_xpm), wxDefaultPosition, wxSize(64, 64))
		, flags);

	szrRight->Add(new wxStaticText(this, wxID_ANY,
		_("Wavtool, Version "WAVTOOL_VERSION
		"\n\n"
		"using "wxVERSION_STRING
		"\n\n"
		"Copyright (C) 2011-2013 Sasaji"))
		, flags);

	wxSizer *szrButtons = CreateButtonSizer(wxOK);
	szrMain->Add(szrLeft, flags);
	szrMain->Add(szrRight, flags);
	szrAll->Add(szrMain, flags);
	szrAll->Add(szrButtons, flags);

	SetSizerAndFit(szrAll);
}
