//
// wavtool.h
//
#ifndef _WAVTOOL_H_
#define _WAVTOOL_H_

#include "common.h"
#include "wx/wx.h"
#include "wx/dnd.h"

#define MAX_BUFFER 32768

class WavtoolApp;
class WavtoolFrame;
class WavtoolPanel;
class WavtoolFileDialog;
class WavtoolFileDropTarget;

class PARSEWAV;
class ConfigBox;

// Window
class WavtoolApp: public wxApp
{
private:
	wxString app_path;

	void SetAppPath();
public:
	bool OnInit();
	int  OnExit();
	wxString GetAppPath();
};

DECLARE_APP(WavtoolApp)

// Frame
class WavtoolFrame: public wxFrame
{
private:
	// gui
	wxMenu *menuFile;
	wxMenu *menuOther;
	wxMenu *menuHelp;
	WavtoolPanel *panel;

	PARSEWAV *wav;

	ConfigBox *cfgbox;

	wxString wav_file_path;
	_TCHAR text_buffer[MAX_BUFFER];

public:

    WavtoolFrame(const wxString& title, const wxSize& size);
	~WavtoolFrame();

	// event procedures
	void OnQuit(wxCommandEvent& event);
	void OnAbout(wxCommandEvent& event);

	void OnOpenFile(wxCommandEvent& event);
	void OnCloseFile(wxCommandEvent& event);

	void OnExportFile(wxCommandEvent& event);

	void OnAnalyzeWave(wxCommandEvent& event);

	void OnOptionRftype(wxCommandEvent& event);
	void OnOptionMachine(wxCommandEvent& event);

	void OnConfigure(wxCommandEvent& event);

	void OnMenuOpen(wxMenuEvent& event);

//	void OnDropFiles(wxDropFilesEvent& event);

	// functions
	void OpenDataFile(wxString &path);
	void CloseDataFile();
	void ExportFile(int id);
	void OpenDroppedFile(wxString &path);

	// properties
	WavtoolPanel *GetWavtoolPanel() { return panel; }

	enum
	{
		// menu id
		IDM_EXIT = 1,
		IDM_ABOUT,
		IDM_OPEN_FILE,
		IDM_CLOSE_FILE,
		IDM_EXPORT,
		IDM_EXPORT_REAL,
		IDM_EXPORT_L3,
		IDM_EXPORT_L3B,
		IDM_EXPORT_L3C,
		IDM_EXPORT_WAV,
		IDM_ANALYZE_WAV,
		IDM_FILE_OPTION,
		IDM_OPTION_RFTYPE,
		IDM_OPTION_MACHINE,
		IDM_CONFIGURE,

		IDD_CONFIGBOX,
	};

	DECLARE_EVENT_TABLE()
};

// Panel
class WavtoolPanel: public wxPanel
{
private:
	wxTextCtrl *textName;
	wxTextCtrl *textInfo;

public:
	WavtoolPanel(WavtoolFrame *parent);

	// event procedures
	void OnSize(wxSizeEvent& event);

	// properties
	wxTextCtrl *GetTextName() { return textName; }
	wxTextCtrl *GetTextInfo() { return textInfo; }


	enum {
		IDC_TEXT_NAME = 1,
		IDC_TEXT_INFO,
	};

	DECLARE_EVENT_TABLE()
};

// for file dialog
class WavtoolFileDialog: public wxFileDialog
{
public:
	WavtoolFileDialog(const wxString& message, const wxString& defaultDir = wxEmptyString, const wxString& defaultFile = wxEmptyString, const wxString& wildcard = wxFileSelectorDefaultWildcardStr, long style = wxFD_DEFAULT_STYLE);

};

// for drop
class WavtoolFileDropTarget : public wxFileDropTarget
{
    WavtoolFrame *frame;
public:
    WavtoolFileDropTarget(WavtoolFrame *parent);
    bool OnDropFiles(wxCoord x, wxCoord y ,const wxArrayString &filenames);
};

// About dialog
class WavtoolAbout : public wxDialog
{
public:
	WavtoolAbout(wxWindow* parent, wxWindowID id);
};

#endif /* _WAVTOOL_H_ */

